%ACTLEVAL Evaluate clustering by active learning performance
%  
%   ERR = ACTLEVAL(A,PROC)
%   ERR = A*ACTLEVAL(PROC)
%
% INPUT
%   A      Labeled PRTools dataset
%   PROC   Cell array of cluster procedure names or mappings, default
%          CLUSTF, CLUSTK, CLUSTH, MEANSHIFT, MODECLUSTF
%
% OUTPUT
%   ERR    Error structure (see PLOTE) containing test errors
%
% DESCRIPTION
% Evaluation of clustering procedures PROC by constructing an active
% learning curve, see MCLUSTLCURVE. Cluster labels are interpreted as
% indices of the labeled dataset A. The resulting cluster labels are
% compared with the true labels of A. ERR refers to the performance on the
% entire dataset and does not distinguish train and test errors.
%
% If no output (E) is requested ACTLEVAL calls PLOTE internally and
% thereby produces automatically a figure with a learning curve.
%
% EXAMPLE
% A = gendatm(3000);
% E = A*actleval({clustm,clustk([],'KCentres')});
% plote(E)
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, CLUSTR, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR,
% CLUSTEVAL, CLUSTCERR, CLUSTC, CLUSTNUM, PLOTE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function err = actleval(varargin)

  argin = shiftargin(varargin,{'cell','prmapping'});
  argin = setdefaults(argin,[],[]);
  if mapping_task(argin,'definition')
    err = define_mapping(argin,'fixed');
  else
    % execution, find input arguments
    [a,proc] = deal(argin{:});
    proc = setdefaults(proc,{clustf,clustk,clusth,meanshift,modeclustf});
    if iscell(proc)
      err = cell(1,numel(proc));
      for i=1:numel(proc)
        err{i} = feval(mfilename,a,proc{i});
      end
      err{1}.title = ['Active learning curve: ' getname(a)];
    elseif ismapping(proc)
      err = mclustlcurve(a*proc,a);
      err.names = getname(proc);
      err.title = ['Active learning curve: ' getname(a)];
    elseif ischar(proc)
      err = mclustlcurve(a,feval(proc,a));
      err.names = proc;
    else
      error('Mapping or name expected for cluster procedure')
    end
    if nargout == 0
      plote(err)
      clear err
    end
  end
  return
