%CLUST2PROTO Find prototypes from given clustering
%
%   LAB_OUT = CLUST2PROTO(LABC,A,TYPE)
%   LAB_OUT = LABC*CLUST2PROTO(A,TYPE)
%   LAB_OUT = CLUST2PROTO(A)
%
% INPUT
%   LABC      Index array, size [M,K], indices of cluster prototypes for M
%             objects in K clusterings.
%   A         M x K PRTools dataset or double array
%   TYPE      String indicating type of desired prototypes: 
%            'medoids' (default) or 'centres'
%
% OUTPUT
%   LAB_OUT  Vector with set of M indices pointing to the nearest
%            prototype for every object in A.
%
% DESCRIPTION
% By this routine cluster labels of a dataset A are replaced by the indices
% of the mediods or the centres of these clusters. In case LAB_IN is not
% given A should be a PRTools dataset and its labels are interpreted as the
% cluster labels.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, CLUSTE, CLUSTF, CLUSTH, CLUSTK, CLUSTKH, CLUSTM, MEANSHIFT, 
% MODECLUST, MODECLUSTF

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com


function out = clust2proto(varargin)

argin = shiftargin(varargin,'last',1);
argin = shiftargin(argin,'char',2);
argin = setdefaults(argin,[],[],'medoids');
if mapping_task(argin,'definition') % facilitates a*clus2proto for PRTools
  out = define_mapping(argin,'fixed');
else % normal call
  [lab,a,type] = deal(argin{:});
  if isempty(lab) % facilitates a*clus2proto for PRTools
    if isa(a,'dataset')
      lab = getnlab(a);
    else
      error('No cluster labels supplied');
    end
  end
  a = +a;
  m = size(a,1);
  if size(lab,1) ~= m
    error('Wrong number of cluster labels supplied');
  end
  nc = size(lab,2);
  out = zeros(m,nc);
  t = sprintf('Processing %i nc clusterings: ',nc);
  prwaitbar(nc,t);
  for j=1:nc                        % run over all clusterings
    prwaitbar(nc,j,[t num2str(j)]);
    labj = lab(:,j);
    clusters = unique(labj);
    k = numel(clusters);
    for i=1:k                       % run over all clusters
      L = find(labj == clusters(i)); % objects in this cluster
      if numel(L) > 1000             % avoid too large datasets
        seed = randreset;
        randreset(1);
        R = randperm(numel(L));
        LL = L(R(1:1000));
        randreset(seed);
      else
        LL = L;
      end
      d = sqrt(distm(a(LL,:)));      % squared distances in cluster
      switch type                   % find cluster prototype
        case 'medoids'
          [~,n] = min(mean(d)); % object with minimum mean sq distance
        case 'centres'
          [~,n] = min(max(d));  % object with minimum max sq distance
        otherwise
          error('Illegal type of prototype requested')
      end
      out(L,j) = repmat(LL(n),numel(L),1); % let all objects point to prototype
    end
  end
  prwaitbar(0);
end
      
    