%CLUSTDIST Cluster distances on the basis of high resolution clustering
%
%  D = CLUSTDIST(LLAB,HLAB)
%
% INPUT
%  LLAB  [M,1] vector with the results of a low resolution clusterings for
%        M objects. 
%  HLAB  [M,1] vector with the results of a high resolution clusterings for
%        M objects. 
%
%  D     Dataset of size NxN with the cluster distances between the N 
%        clusters given in LLAB. The labels of this dataset are the cluster
%        labels as given in LLAB.
%
% DESCRIPTION
% First sumilarities [0,1] are computed between low resolution clusters on  
% the basis of the total numbers of objects of high resolution clusters in
% one of the low resolution clusters for which the high resolution cluster
% is in the othe low resolution cluster. Next these similarities are
% converted to distances. 
%
% Note that two remote low resolution clusters may not share a single high
% resolution cluster, resulting in a similarity 0 and a maximum distance 1.
% The similarity between a cluster and itself is 1, resulting in a distance
% of 0.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, CLUSTM

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function d = clustdist(L,H)

if size(L,1) ~= size(H,1)
  error('Cluster labelings should have equal size')
end

if size(L,2) ~= 1
  error('Multilevel clustering not supported')
end

P = unique(L);         % low-resolution prototypes
N = numel(P);          % their number --> number of clusters
s = zeros(N,N);        % space for bordersize matrix

if size(H,2) > 1
  r = size(H,2);
  d = zeros(N,N);
  for i=1:r
    d = d+feval(mfilename,L,H(:,i));
  end
  d = d/r;
else
  t = sprintf('Computing distances for %i clusters: ',N);
  prwaitbar(N,t);
  for j=1:N
    prwaitbar(N,j,[t num2str(j)]);
    Pj = L == P(j);        % objects in cluster j
    R = unique(H(Pj));     % high-resolution protoypes in that cluster
    for i=1:N
      Pi = L == P(i);      % objects in cluster i
      s(i,j) = sum(ismember(H(Pi),R));% # objects in Pi from prototypes in Pj
    end
  end
  prwaitbar(0);

  % normalize d: distance between two clusters is one minus the relative 
  % size of their borders
  d = zeros(size(s));
  for j=1:N
    for i=1:N
      d(i,j) = 1 - (s(i,j)+s(j,i))/(s(i,i)+s(j,j));
    end
  end
  d = prdataset(d,P);
end
