%CLUSTE Wrapper for examplar clustering by DCLUSTE for feature based data
%
% 	LAB = CLUSTE(A,K,MSIZE)
%   LAB = A*CLUSTE(K,MSIZE)
%
% INPUT
%   A    Feature based dataset or double array with M objects (rows).
%   K    Scalar or a vector of length N with desired numbers of clusters.
%        Default is a set of N clusterings with numbers that naturally
%        arise from the data.
%  MSIZE Number of objects (M) above which processing is automatically 
%        taken over by CLUSTME (default 2000). Use MSIZE = inf to avoid use
%        of CLUSTME.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% This routine performs a clustering based on message passing between data
% points, see [1]. It is a wrapper around DCLUSTE which is based on 
% EXEMPLAR. That routine should be called directly to set other parameters
% than its defaults.
%
% EXEMPLAR does not return clusterings with a preset number of clusters K.
% Its multilevel clustering is first made nested by RECLUSTN. Next the
% desired number of clusters is realized by RECLUSTK.
%
% EXAMPLE
% randreset;                     % take care of reproducability
% data = gendatclust1(20000);    % generate 20000 objects in 10 clusters
%                                % Run Mean Shift clustering
% lab = cluste(data,[2 5 10 18 30 50 100],2000);
%                                % Show scatterplot for 10 clusters
% figure; scatn(lab(:,3),data,'Exemplar'); 
% figure; clusteval(lab,data);   % Evaluation by active learning
% 
% REFERENCES
%   [1] B.J. Frey and D. Dueck, Clustering by passing messages between
%       data points, Science, vol. 315, pp. 972-976, 2007
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTH, CLUSTH, CLUSTF CLUSTK, CLUSTM, CLUSTME,
% CLUSTKE, MODECLUSTF, CLUSTEVAL, CLUSTCERR, CLUSTC, CLUSTNUM, CLUSTHC,

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = cluste(varargin)

  argin = shiftargin(varargin,'vector',1);
  argin = setdefaults(argin,[],[],2000);
  if mapping_task(argin,'definition')
    % definition of the mapping
    labels = define_mapping(argin,'fixed','Exemplar');
  else
    % execution, find input arguments
    [a,K,msize] = deal(argin{:});
    a = +a;
    if size(a,1) >= msize
      % preclustering needed
      thisclust = feval(mfilename,[],K,msize);
      labels = preclust(a,thisclust,clustm,msize);
      if ~isempty(K)
        labels = reclustk(labels,K);
      end
    else
      K = K(K<=msize);
      d = sqrt(distm(a));
      labels = dcluste(d,K);
    end
  end
return