%CLUSTEVAL Evaluate clusterings by various performance measures
%
%     E = CLUSTEVAL(LABC,LABT,TYPE)
%     E = CLUSTEVAL(LABC,A,TYPE)
%     E = LABC*CLUSTEVAL(LABT,TYPE)
%     E = LABC*CLUSTEVAL(A,TYPE)
%
% INPUT
%   LABC   Index array, size [M,N], indices of cluster prototypes for M
%          objects in N clusterings.
%   LABT   Double array, size [M,1] with true object labels.
%   A      PRTools dataset used for obtaining LABC by some clustering
%          procedure.
%   TYPE   String with desired performance measure, see below.
%
% OUTPUT
%   E      Evaluation result using the performance measure give by TYPE.
%          See below for possibilities.
%
% DESCRIPTION
% Computation of a set of cluster performance measures between estimated
% cluster labels LABC and true object labels LABT. In case LABC is a
% multilevel clustering (N>1) the result E is a structure ready to be
% plotted by PLOTE. If E is omitted (no output) the result is directly
% plotted. Performance measures that do not generate a curve (see below) 
% are plotted on the screen.
%
% In case LABC is a single clustering (N==1) just scalar results are
% returned (except for TYPE is 'roc', which generates two values, see
% CLUSTROC). In case TYPE is omitted the default measure for multilevel
% clustering is 'actl'. For a single clustering all measures are returned
% in a structure or printed on the screen.
%
% It is assumed that the cluster labels LABC are indices to cluster
% prototypes with true labels as given by the correponding entries in LABT.
% The true labels LABT can be derived as doubles from a PRTools dataset A
% by LABT = GETNLAB(A); 
%
% The following performance measures are available;
% ROC    - Relative operating characteristics, see CLUSTROC.
% AUC    - Scalar, area under the ROC curve [e1,e2].
% MINE   - Scalar, MIN(E1+E2) with E1 and E2 the two errors of the ROC.
% ACTL   - Classification error of assigning all objects to the true class
%          of the cluster prototypes (active learning).
% COMB   - Classification error based on the true class of the cluster
%          prototypes and combining multilevel cluster confidences.
% NEST   - Classification error based on the true class of the cluster
%          prototypes after nested combining cluster levels, see RECLUSTN
% ADRI   - The adjusted Rand index, see <a href="https://en.wikipedia.org/wiki/Rand_index#Adjusted_Rand_index">Wikipedia</a>.
%          It is between 0 and 1 and 1 for consistent clusterings.
% NMI    - Normalised mutual information between 0 and 1. 
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, CLUSTR, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR,
% RECLUSTN, CLUSTCERR, CLUSTC, CLUSTNUM, CLUSTROC PLOTE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function out = clusteval(varargin)

argin = shiftargin(varargin,'prdataset',1);
argin = shiftargin(argin,'last',1);
argin = shiftargin(argin,'char',2);
argin = setdefaults(argin,[],[],[]');
if mapping_task(argin,'definition')
  out = define_mapping(argin,'fixed');
else
  
  % check inputs
  [labc,a,type] = deal(argin{:});
  [labt,datname] = truelabs(a);
  if isempty(type)
    if size(labc,2) > 1
      type = 'actl';
    else
      type = 'all';
    end
  end
  m = size(labc,1);
%   if isdataset(a)
%     labt = getnlab(a);
%     datname = getname(a);
%   else
%     labt = a;
%     datname = '';
%   end
  if numel(labt) ~= m
    error('Inputs should refer to the same number of objects')
  end

  % the real work
  if size(labc,2) == 1      % handle single clustering
    switch lower(type)
      case 'roc'
        e = clustroc(labc,labt);
        e = [e.error(2),e.xvalues(2)];
        if nargout == 0
          fprintf('%s = %5.3f %5.3f\n',upper(type),e(1),e(2));
        else
          out = e;
        end
      case {'auc','mine'}
        if nargout == 0
          feval(mfilename,[ones(m,1) labc [1:m]'],labt,type);
        else
          out = feval(mfilename,[ones(m,1) labc [1:m]'],labt,type);
        end
      case 'actl'
        e = clustcerr(labc,labt);
        if nargout == 0
          fprintf('%s = %5.3f\n',upper(type),e);
        else
          out = e;
        end
      case {'adri','nmi'}
        e = feval(mfilename,[ones(m,1) labc [1:m]'],labt,type);
        if nargout == 0
          fprintf('%s = %5.3f\n',upper(type),e.error(2));
        else
          out = e.error(2);
        end
      case 'all'
        all = {'roc','auc','mine','actl','adri','nmi'};
        out = struct;
        for n=1:numel(all)
          out.(all{n}) = NaN;
        end
        for n=[1 4:numel(all)]
          out = setfield(out,all{n},feval(mfilename,labc,labt,all{n}));
        end
        [e1,e2] = deal(out.roc(1),out.roc(2));
        out.auc  = e1*e2 + e1*(1-e2)/2 + e2*(1-e1)/2;
        out.mine = e1+e2;
        if nargout == 0
          fprintf('%s = %5.3f %5.3f\n','ROC',e1,e2);
          for n=2:numel(all)
            fprintf('%s = %5.3f\n',upper(all{n}),out.(all{n}));
          end
          clear out;
        end
      otherwise
        error('Unknown performance measure')
    end
    
  else % multilevel clustering
      
    labc = labsort(labc,'up');
    N = clustnum(labc);
    e.xlabel = 'Number of clusters';
    if max(N) <= 100
      e.plot = 'plot';
    else
      e.plot = 'semilogx';
    end

    switch lower(type)
      case {'roc','auc','mine'}
        e = clustroc(labc,labt);
        [e1,e2] = deal(e.error,e.xvalues);
        switch lower(type)
          case 'auc'
            auc = 0;
            for j=2:numel(e1)
              auc = auc + ((e2(j)+e2(j-1))/2) * (e1(j)-e1(j-1));
            end
            e = auc;
          case 'mine'
            e = min(e1+e2);
        end
      case 'comb'
        e = mclustlcurve(labc,labt);
        %e.title = ['Active Learning by Cluster Combining'];
      case 'nest'
        labn = reclustn(labc);
        e = clustlcurve(labn,labt);
        e.title = ['Nested Active Learning Curve'];
      case 'actl'
        e = clustlcurve(labc,labt);
        e.title = ['Active Learning Curve'];
      case 'adri'
        e.error   = adjrand(labc,labt);
        e.xvalues = N;
        e.xlabel  = 'Number of clusters';
        e.ylabel  = 'Adjusted Rand Index';
        e.title   = ['Clustering performance'];
      case 'nmi'
        e.error   = labnmi(labc,labt);
        e.xvalues = N;
        e.xlabel  = 'Number of clusters';
        e.ylabel  = 'Normalised Mutual Information';
        e.title   = ['Clustering performance'];
      otherwise
        error('Unknown performance measure')
    end
    
    if nargout == 0                                  % no output requested
      if strcmpi(type,'auc') || strcmpi(type,'mine') % print on screen
        fprintf('%s = %5.3f\n',upper(type),e);
      else
        plote(e);                                    % create plot
      end
    else
      out = e;
    end
  
  end

end
return

function adri = adjrand(labc,labt)

%function ADRI = adjrand(LABC,LABT)
%
% Computes the adjusted Rand index to assess the quality of a clustering.
% Perfectly random clustering returns the minimum score of 0, perfect
% clustering returns the maximum score of 1.
%
%INPUTS
% LABC = the labeling as predicted by a clustering algorithm
% LABT = the true labeling
%
%OUTPUTS
% ADRI = the adjusted Rand index
%
%
%Author: Tijl De Bie, february 2003.

u = labc;
v = labt;

if size(u,2) > 1
  adri = zeros(1,size(u,2));
  for j=1:size(u,2)
    adri(1,j) = feval('adjrand',labc(:,j),labt);
  end
  return
end

n=length(u);
ku=max(u);
kv=max(v);
m=zeros(ku,kv);
for i=1:n
    m(u(i),v(i))=m(u(i),v(i))+1;
end
mu=sum(m,2);
mv=sum(m,1);

a=0;
for i=1:ku
    for j=1:kv
        if m(i,j)>1
            a=a+nchoosek(m(i,j),2);
        end
    end
end

b1=0;
b2=0;
for i=1:ku
    if mu(i)>1
        b1=b1+nchoosek(mu(i),2);
    end
end
for i=1:kv
    if mv(i)>1
        b2=b2+nchoosek(mv(i),2);
    end
end

c=nchoosek(n,2);

adri=(a-b1*b2/c)/(0.5*(b1+b2)-b1*b2/c);

return