%CLUSTF Clustering by the FFT (Farthest First Traversal) algorithm
%
%  LAB = CLUSTF(A,K.MSIZE)
%  LAB = A*CLUSTF(K.MSIZE)
%
% INPUT
%   A    Feature based dataset with M objects.
%   K    Vector with desired numbers of clusters, default sampling of [2:M]
% MSIZE  Number of objects (M) above which the dataset is preclustered by
%        CLUSTM, reducing it to MSIZE objects. Default MSIZE = 5000. Use
%        MSIZE = inf to avoid perclustering.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% The dataset A with M rows (objects) is clustered by the Farthest First 
% Traversal algorithm, initialised by the object nearest to the mean of A.
% In every step a next cluster prototype is defined as the not yet selected
% object in A that is farthest to the existing set of prototypes.
% Objects are clustered by assigning them to the nearest prototype. The
% dataset A can be either a double array or a PRTools dataset. Its labels
% are negelected.
%
% In LAB for all objects the index in A of the nearest cluster prototype is
% returned. If K is a vector LAB has length(K) columns, returning a 
% multilevel clustering. 
%
% If M > MSIZE the dataset A is preclustered by PRECLUST using CLUSTM.
% Unless specific values of K < 100 are needed it is recommended for fast
% processing to use K = []. Speed may be further increased by using smaller
% values of MSIZE, e.g. MSIZE = 500;
%
% EXAMPLE
% randreset;                     % take care of reproducability
% data = gendatclust1(20000);    % generate 20000 objects in 10 clusters
%                                % Run Mean Shift clustering
% lab = clustf(data,[2 5 10 18 30 50 100],2000);
%                                % Show scatterplot for 10 clusters
% figure; scatn(lab(:,3),data,'FFT'); 
% figure; clusteval(lab,data);   % Evaluation by active learning
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM, CLUSTS,
% CLUSTR, CLUSTEVAL, CLUSTCERR, CLUSTC, CLUSTNUM, PRECLUST

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function lab = clustf(varargin)

  argin = shiftargin(varargin,'vector',1);
  argin = setdefaults(argin,[],[],5000);
  if mapping_task(argin,'definition')
    % mapping definition
    lab = define_mapping(argin,'fixed','FFT');
  else
    % execution, find input arguments
    [a,K,msize] = deal(argin{:});
    a = +a;
    if size(a,1) >= msize
      % preclustering needed
      thisclust = feval(mfilename,[],K,msize);
      lab = preclust(a,thisclust,clustm,msize);
      if ~isempty(K)
        lab = reclustk(lab,K);
      end
    else
      m = size(a,1);                      % number of objects
      K = setdefaults(K,intgeos(2,1.21,m/5));% default numbers of clusters
      lab = zeros(m,numel(K));            % space for all labelings
      [p,J] = selproto(a,max(K),'fft');   % Find the prototypes
      for i = 1:numel(K)                  % Store the desired labelings
        lab(:,i) = J(labnn(a,p(1:K(i),:)));
      end
    end
  end
  
  



