%CLUSTHC Classifier for hierarchical clustering
%
%    W = CLUSTHC(A,TYPE)
%    W = A*CLUSTHC(TYPE);
%    LAB = B*W*LABELD
%
% INPUT
%   A    Dataset, typically labeld with cluster labels obtained by a
%        hierarchical clustering of the given TYPE, see CLUSTH.
%  TYPE  Linkage rule: 's', 'a', 'r' or 'c'. Alternatively
%        'single','average','central' or 'complete'. Default 'single'.
%   B    New objects (dataset or doubles) to be assigned to the clusters
%        in A.
%
% OUTPUT
%   LAB  Estimated labels of B.
%   W    Classifier(s) assigning new objects B using the linkage rule 
%        defined by TYPE.
%
% DESCRIPTION
% This routine is useful to find a classifier for assigning new objects to
% clusters found by a hierarchical clustering.
%
% It is assumed that A is a dataset of which the labels refer to clusters
% found by a hierarchical cluster procedure CLUSTH with linkage type TYPE.
% The classifier W assigns new objects B to the most neighboring cluster
% according to the cluster distance defined by TYPE.
%
% EXAMPLE
% x = +gendatclust(1000);
% scattern(x);
% lab = x*clusth(10);
% w = prdataset(x,lab)*clusthc;
% plotc(w);
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, LABELD, CLUSTH, LABELD

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function out = clusthc(varargin)
	
  argin = shiftargin(varargin,'char',1);
  argin = setdefaults(argin,[],'single');
  if mapping_task(argin,'definition')
    name = linktype(argin{2});
    % definition of the mapping in case of A*CLUSTK([],PARS)
    out = define_mapping(argin,'fixed',name);
    
  elseif mapping_task(argin,'training')
    
    [a,type] = deal(argin{:});
    out = trained_classifier(a,{a,type})*classc;
    
  elseif mapping_task(argin,'execution')
    
    [testset,w] = deal(argin{1:2});
    [trainset,type] = getdata(w,1,2);
    [~,type] = linktype(type);
    lablist = getlablist(trainset);
    c = getsize(trainset,3);
    dclass = zeros(size(testset,1),c);
    if strcmp(type,'central') || strcmp(type,'r')
      d = sqrt(+distm(testset,trainset));
      dclass = d(:,lablist);
    else
      for j=1:c
        classj = seldat(trainset,j);
        dj = +distm(testset,classj);
        switch lower(type)
          case {'a','average'}
            dclass(:,j) = mean(dj,2);
          case {'c','complete'}
            dclass(:,j) = max(dj,[],2);
          case  {'s','single'}
            dclass(:,j) = min(dj,[],2);
          otherwise
            error('Unknown linkage type')
        end
      end
    end
    dclass = (repmat(sum(dclass,2),1,c)-dclass)*normm;
    out = setdata(testset,dclass);
    if isdataset(out)
      out = setfeatlab(out,lablist);
    end
  end
  