%CLUSTK Feature space centroid-based clustering with K prototypes: 
%       kmeans, kcentres and kmedoids
% 
%   LAB = CLUSTK(A,K,TYPE,R,MSIZE)
%   LAB = A*CLUSTK(K,TYPE,INIT,R,MSIZE)
% 
% INPUT
%   A    Feature based dataset with M objects, possibly doubles.
%   K    Vector with desired numbers of clusters, default sampling of [2:M]
%  TYPE  'kmeans' (default), 'kcentres' or 'kmedoids'.
%  INIT  Vector of length max(K), indices of initial centres or medoids.
%        INIT = []: default: systematic initialisation by CLUSTF.
%   R    Number of clustering trials based on random initialisations. 
%        The best cluster result is returned.
% MSIZE  Number of objects (M) above which the dataset is preclustered by
%        CLUSTM, reducing it to MSIZE objects. Default MSIZE = 5000. Use
%        MSIZE = inf to avoid perclustering.
%
% OUTPUT
%   LAB  M*NUMEL(K) array with the results of the multilevel clustering for
%        the M objects. The columns refer to the clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% An intial set of K prototypes is iterativly optimised such that the
% set of objects with the same nearest prototype (a 'cluster') constitutes
% this protoype exactly as its mean, medoid or centre. The medoid is
% defined as that object in a cluster for which the mean distance to all
% other objects is minimum. The centre is defined as that object in the a
% cluster for which the maximum distance to all other objects is minimum.
%
% The clustering is iterated untill stability or is prematurely stopped by
% PRTIME. In case of random initialisations (R is an integer > 0) the
% clustering is repeated R times and the best result is returned. For R = 0
% (special case) a systematic initialisation is performed and the resulting
% clustering is directly returned without optimisation.
%
% LAB is a column vector of length M or an array of length(K) columns. It 
% contains for every object and for every clustering the cluster indices. 
% In case of kcentres or kmedoids they point to the objects that are found
% as the centres or prototypes. In case of kmeans they point to the objects
% nearest to the cluster means.
%
% If K is given its values are reduced to less than M/5 to make the routine
% more feasible. Moreover, if M > MSIZE the dataset A is preclustered by
% PRECLUST using CLUSTM. Unless specific values of K < 100 are needed it is
% recommended for fast processing to use K = []. Speed may be further
% increased by using smaller values of MSIZE, e.g. MSIZE = 500;
%
% EXAMPLE
% randreset;                     % take care of reproducability
% data = gendatclust1(20000);    % generate 20000 objects in 10 clusters
%                                % Run Mean Shift clustering
% lab = clustk(data,[2 5 10 18 30 50 100],'kmeans',[],2000);
%                                % Show scatterplot for 10 clusters
% figure; scatn(lab(:,3),data,'K-Means'); 
% figure; clusteval(lab,data);   % Evaluation by active learning
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTH, CLUSTE, CLUSTF CLUSTH, CLUSTKH, CLUSTM,
% RECLUSTN, PRECLUST, CLUSTEVAL, CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = clustk(varargin)

  argin = shiftargin(varargin,{'vector'},1);
  argin = shiftargin(argin,{'char'},2);
  argin = setdefaults(argin,[],[],'KMeans',[],5000);
  name = argin{3};
  if mapping_task(argin,'definition')
    % definition of the mapping in case of A*CLUSTK([],PARS)
    labels = define_mapping(argin,'fixed',name);
  else
    % execution, find input arguments
    [a,K,type,N,msize] = deal(argin{:});      % get inputs
    a = +a;
    if size(a,1) >= msize
      % preclustering needed
      thisclust = feval(mfilename,[],K,type,N,msize);
      labels = preclust(a,thisclust,clustm,msize);
      if ~isempty(K)
        labels = reclustk(labels,K);
      end
    else
      % here for datasets smaller than msize objects
      [m,nfeat] = size(a);     % size of the data
      type = clustertype(type);
      if isempty(K)
      	K = intgeos(2,1.21,size(a,1)/5);
        emptyK = true;
      else
        emptyK = false;
      end
      K = K(K<size(a,1)/5);
      [is_means,is_centres,is_medoids] = deal(false,false,false);
      switch lower(type)
        case 'kmeans'
          is_means = true;
        case 'kcentres'
          is_centres = true; 
        case 'kmedoids'
          is_medoids = true;
        otherwise
          error('Unknown cluster type')
      end
      nk = numel(K);           % number of desired clusterings
      if nk > 1                % if more than one, make a loop
        labels = zeros(m,nk);  % space for resulting cluster labels
        t = sprintf('Performing %i clusterings: ',nk);
        prwaitbar(nk,t);
        for j=1:nk             % loop to compute all clusterings
          prwaitbar(nk,j,[t num2str(j)]);
          labels(:,j) = feval(mfilename,a,K(j),type,N,msize);
        end
        prwaitbar(0);
        if emptyK
          labels = reclustn(labels); % nest the clusters
        end
      else
        k = K;        % we have to perform a single clustering 
        if (k == 1)   % k=1 is a special case, find mean, centre or medoid
          if is_means
            j = 1;
          elseif is_centres
            j = minmaxdist(a);    
          elseif is_medoids
            j = minmeandist(a); 
          else
            error('Unexpected error')
          end
          labels = repmat(j,m,1);  % give all objects the same label
        elseif (k > 0)
          k = min(k,m);            % k should not be larger than sample size
          updating = true;
          if isempty(N) || (numel(N)==1 && N==0)
  %           % use systematic initialisation
  %           M = unique(clustf(a,k));
            if m > 1000 
              seed = randreset(1);
              if k < 1000
                seed = randreset(1);
                R = randperm(m)';
                d = distm(a(R(1:1000),:));
                M = dselproto(d,k,type);  % use systematic initialisation
                M = R(M);
                randreset(seed);
              else
                M = randperm(m,k);
              end
              randreset(seed);
            else
              M = dselproto(distm(a),k,type); % use systematic initialisation
            end
            ntrial = 1;
            if N == 0  % arrange systematic initialisation only
              updating = false; 
              N = []; 
            end
          elseif length(N) == 1           % random initialisation
            ntrial = N;                   % set number of trials
          else                            % initialisation by object numbers
            if k > numel(N)               % we should be able to find k      
              error('Initialisation should contain at least k indices')
            elseif max(N) > m
              error('Initialisation out of range')
            end
            M = N(1:k);                   % object indices for initialisation 
            ntrial = 1;       
          end
          opt_perf = inf;
          s = sprintf([name ', %i trials: '],ntrial);
          prwaitbar(ntrial,s);
          for tri = 1:ntrial
            prwaitbar(ntrial,tri,[s int2str(tri)]);
            if numel(N) == 1 % random initialisation desired
              M = randperm(m)'; M = M(1:k);		% Random initializations
            end
            p = a(M,:);                  % initial set of prototypes
            if (is_means), M = (1:k)'; end
            labels_old = zeros(m,1);

            starttime = clock;
            runtime = 0;
            iter = 0;
            t = sprintf('Clustering, iteration: ');
            prwaitbar(prtime,t);
            while runtime < prtime  % Iterate until no change or end of time
              iter = iter+1; 
              prwaitbar(prtime,runtime,[t num2str(iter)]);
              %[~,I] = min(distm(a,p),[],2); % nearest prototype
              I = indnn(a,p);              % nearest prototype
              labels_new = M(I);          % find prototype indices
              Iproto = unique(labels_new); % cluster labels (prototype indices)
              k = numel(Iproto);           % number of clusters active

              % Redefine prototypes for clusters. 
              J = zeros(k,1);        % space for updated prototype indices
              p = zeros(k,nfeat);    % space for updated prototypes
              for i = 1:k            % run over clusters                
                JJ = find(labels_new==Iproto(i));  % find objects in cluster
                % find for every cluster) its mean or prototype              
                if is_means
                  p(i,:) = mean(a(JJ,:),1);
                  J(i) = i;
                elseif is_centres
                  j = minmaxdist(a(JJ,:));
                  p(i,:) = a(JJ(j),:);   
                  J(i) = JJ(j);
                elseif is_medoids
                  j = minmeandist(a(JJ,:));
                  p(i,:) = a(JJ(j),:);
                  J(i) = JJ(j);
                else
                  error('unexpected error')
                end
              end

              if (length(M) == k) && ...
                  (((is_centres || is_medoids) && all(M == J)) || ...
                  ((is_means) && all(labels_old == labels_new)) || ~updating)
                % prototypes are found, we are done
                break;
              end
              M = J;                   % prototype indices 
              labels_old = labels_new; % all labels
              runtime = etime(clock,starttime);
            end
            if runtime > prtime
              prwarning(2,['k-cluster updating stopped by PRTIME after ' num2str(iter) ' iterations']);
            end
            % compute performances
            if ntrial > 1
              if is_centres
                perf = maxmindist(a,p);
              else % means, medoids
                perf = meanmindist(a,p);
              end

              % Store the optimal prototypes in JOPT.
              if (perf < opt_perf)
                opt_perf = perf;
                labels = labels_new;
              end
            else
              labels = labels_new;
            end
            prwaitbar(0);
          end
          if is_means
            labels = clust2proto(labels,a);
          end
          prwaitbar(0)
        else
          error(['Desired number of clusters should be positive and' ...
                  newline 'may not exceed number of objects'])
        end
      end
    end
  end
return


function j = minmaxdist(a)

  m = size(a,1);
  batch = ceil(prmemory/(20*size(a,1)));
  dmax = zeros(m,1);
  n = 0;
%   t = sprintf('Finding centre of %i objects: ',m);
%   prwaitbar(m,t)
  for i=1:floor(m/batch)
    dmax(n+1:n+batch) = max(distm(a(n+1:n+batch,:),a),[],2);
    n = n+batch;
%     prwaitbar(m,n,[t num2str(n)]);
  end
  dmax(n+1:m) = max(distm(a(n+1:m,:),a),[],2);
%   prwaitbar(0);
  [~,j] = min(dmax);
  
return

function j = minmeandist(a)

  m = size(a,1);
  batch = ceil(prmemory/(20*size(a,1)));
  dmean = zeros(m,1);
  n = 0;
%   t = sprintf('Finding medoid of %i objects: ',m);
%   prwaitbar(m,t)
  for i=1:floor(m/batch)
    dmean(n+1:n+batch) = mean(sqrt(distm(a(n+1:n+batch,:),a)),2);
    n = n+batch;
%     prwaitbar(m,n,[t num2str(n)]);
  end
  dmean(n+1:m) = mean(sqrt(distm(a(n+1:m,:),a)),2);
%   prwaitbar(0);
  [~,j] = min(dmean);
  
return

function d = meanmindist(a,p)

  m = size(a,1);
  batch = ceil(prmemory/(20*size(p,1)));
  dmin = zeros(m,1);
  n = 0;
%   t = sprintf('Finding nearest neighbors of %i objects: ',m);
%   prwaitbar(m,t)
  for i=1:floor(m/batch)
    dmin(n+1:n+batch) = min(distm(a(n+1:n+batch,:),p),[],2);
    n = n+batch;
%     prwaitbar(m,n,[t num2str(n)]);
  end
  dmin(n+1:m) = min(distm(a(n+1:m,:),p),[],2);
%   prwaitbar(0);
  d = mean(dmin);
  
return

function d = maxmindist(a,p)

  m = size(a,1);
  batch = ceil(prmemory/(20*size(p,1)));
  dmin = zeros(m,1);
  n = 0;
%  t = sprintf('Finding nearest neighobrs of %i objects: ',m);
%   prwaitbar(m,t)
  for i=1:floor(m/batch)
    dmin(n+1:n+batch) = min(distm(a(n+1:n+batch,:),p),[],2);
    n = n+batch;
%    prwaitbar(m,n,[t num2str(n)]);
  end
  dmin(n+1:m) = min(distm(a(n+1:m,:),p),[],2);
%  prwaitbar(0);
  d = max(dmin);
  
return