%CLUSTLCURVE Learning curve from multilevel clustering for active learning
%
%  E = CLUSTLCURVE(LABC,LABT)
%  E = CLUSTLCURVE(LABC,A)
%  E = LABC*CLUSTLCURVE(LABT)
%  E = LABC*CLUSTLCURVE(A)
%
% INPUT
%   LABC   Index array, size [M,K], indices of cluster prototypes for M
%          objects in K clusterings.
%   LABT   Vector of M elements with true object labels.
%          Default: true labels used in some other routine.
%   A      PRTools labeled dataset used for the clustering.
%
% OUTPUT
%   E      Structure with classification error and numbers of labeled
%          objects, ready to plot by PLOTE
%
% DESCRIPTION
% This routines calls CLUSTCERR to create a learning curve ready to be plot 
% by PLOTE. This learning curve evaluates active learning in which objects 
% are assigned to the class of cluster prototypes.
%
% If no output (E) is requested CLUSTLCURVE calls PLOTE internally and
% thereby produces automatically a figure with a learning curve.
%
% EXAMPLE
% dset = gendatm(30000);
% labc = dset*clustm;
% clustlcurve(labc,dset)
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, CLUSTR, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR,
% CLUSTEVAL, CLUSTC, CLUSTNUM, CLUSTCERR, PLOTE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function e = clustlcurve(varargin)

  argin = shiftargin(varargin,'last',1);
  argin = setdefaults(argin,[],[]);
  if mapping_task(argin,'definition')
    e = define_mapping(argin,'fixed');
  else
    [labe,labt] = deal(argin{:});
    labe = labsort(labe,'upred');
    [labt,name] = truelabs(labt);
    [err,sizes] = clustcerr(labe,labt);

    e.error = err;
    e.xvalues = sizes;

    e.title    = ['Active learning curve ' name];
    e.xlabel   = 'Number of labeled objects (clusters)';
    e.ylabel   = 'Classification error';
    e.plot     = 'plot';
    if max(sizes)/min(sizes) >= 100
      e.plot = 'semilogx';
    end
    if nargout == 0
      plote(e);
    end
  end

return