%CLUSTR Random clustering
%
%  LAB = CLUSTR(A,K)
%  LAB = A*CLUSTR(K)
%
% INPUT
%   A    Feature based dataset with M objects.
%   K    Vector with desired numbers of clusters, default sampling of [2:M]
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% The dataset A with M rows (objects) is clustered by selecting a random
% set of K prototypes out of A. All other objects are assigned to the
% nearest prototype. In case K is a set, larger sets of prototypes will
% contain the smaller ones (nested result).
%
% The classifiers are based on the nearest neighbor classifier (KNNC)
% trained by the prototypes found in the clustering using KNNC. Its output
% labels are the indices in A of these objects.
%
% EXAMPLE
% randreset;                     % take care of reproducability
% data = gendatclust1(20000);    % generate 20000 objects in 10 clusters
%                                % Run Mean Shift clustering
% lab = clustr(data,[2 5 10 18 30 50 100]);
%                                % Show scatterplot for 10 clusters
% figure; scatn(lab(:,3),data,'Random'); 
% figure; clusteval(lab,data);   % Evaluation by active learning
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR, CLUSTEVAL,
% CLUSTCERR, CLUSTC, CLUSTNUM

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [lab,w] = clustr(varargin)

  argin = shiftargin(varargin,'vector',1);
  argin = setdefaults(argin,[],[],false);
  if mapping_task(argin,'definition')
    % definition of the mapping in case of A*CLUSTR([],PARS)
    lab = define_mapping(argin,'fixed');
  else
    % execution, find input arguments
    [a,K,flag] = deal(argin{:});
    a = +a;                             % get rid of PRTools labeling 
    m = size(a,1);                      % number of objects
    K = setdefaults(K,intgeos(2,1.21,m/10));% default numbers of clusters
    lab = zeros(m,numel(K));            % space for all labelings
    [p,J] = selproto(a,max(K),'random');% Find the prototypes
    for i = 1:numel(K)                  % Store the desired labelings
      lab(:,i) = J(labnn(a,p(1:K(i),:)));
    end
    if nargout > 1 && flag              % cell array of classifiers desired
      w = cell(1,numel(K));
      for i = 1:numel(K)                % run over desired clusterings
        J = unique(lab(:,i));           % result with K(i) clusters
        w{i} = knnc(prdataset(a(J,:),lab(J,end))); % use it for 1NN rule
      end
    elseif nargout > 1                  % just the final classifier desired
      J = unique(lab(:,end));           % the final clustering
      w = knnc(prdataset(a(J,:),lab(J,end))); % use it for 1NN rule
    end
  end
  
  



