%CLUSTROC Relative operating characterics of multilevel clustering
%
%    [E1,E2,N] = CLUSTROC(LABC,LABT)
%    [E1,E2,N] = CLUSTROC(LABC,A)
%     ROC      = CLUSTROC(LABC,LABT)
%     ROC      = CLUSTROC(LABC,A)
%
% INPUT
%   LABC  Index array, size [M,K], indices of cluster prototypes for M
%         objects in K clusterings.
%   LABT  Double arrys, size [M,1] with true object labels.
%   A     Labeled PRTools dataset with M objects. It used for obtaining
%         the clusterings stored in LABC. LABT = GETNLAB(A)
%
% OUTPUT
%   E1    Double array, size [1,NUMEL(N)], fraction of object pairs
%         erroneously assigned to the same cluster (Error I in ROC plot).
%   E2    Double array, size [1,NUMEL(N)], fraction of object pairs
%         erroneously assigned to different clusters (Error II in ROC plot).
%   N     Row vector with K or less elements with the corresponding number
%         of clusters in the clustering used for the vectors E1 and E2.
%   ROC   Relative operating characteristic, ready to be plotted by PLOTE.
%   
%
% DESCRIPTION
% This routine is used for some performance measures used in CLUSTEVAL.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM, CLUSTF, CLUSTR,
% DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR, CLUSTEVAL, PLOTE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [e1,e2,n,R] = clustroc(labc,labt)   

  if isdataset(labt), 
    datname = getname(labt);
    labt = getnlab(labt); 
  else
    datname = '';
  end
  labc  = labsort(labc,'upred');
  [m,k] = size(labc);
  err1  = zeros(1,k);
  err2  = zeros(1,k);
  n = clustnum(labc);
  n1 = 0;
  n2 = 0;

  % the following tricky loop compares all objects with all objects for
  % all clusterings
  seed = randreset(1);
  R = randperm(m);
  starttime = clock;
  runtime = 0;
  r = 0;
  t = sprintf('Clustering performance estimation, iteration: ');
  prwaitbar(prtime,t);

  while r < m && runtime < prtime
    % we randomly take objects as far as time permits
    r = r+1;
    j = R(r);
    prwaitbar(prtime,runtime,[t num2str(r)]);
    % take an object
    JT = (labt == labt(j)); % find other objects with the same true label
    ms = sum(JT);            % their number
    % find how often this does not hold for the k cluster labels
    err1 = err1 + sum(labc(JT,:) ~= repmat(labc(j,:),ms,1));
    % get counter correct
    n1 = n1 + ms - 1; % correction for comparing point j with point j

    % repeat the same of all objects with a different label than the present
    JT = ~JT;
    md = sum(JT);   % their number
    % find how often this does not hold for the k label estimates
    err2 = err2 + sum(labc(JT,:) == repmat(labc(j,:),md,1));
    % counter
    n2 = n2 + md;
    runtime = etime(clock,starttime);
  end
  if runtime > prtime
    prwarning(3,['Clustering performance estimation stopped by PRTIME at: ' ...
      num2str(n1+n2) ' pairs']);
  end
  prwaitbar(0);
  randreset(seed);

  % add the asymptotic solutions
  e1 = [0 err1/n1 1]';
  e2 = [1 err2/n2 0]';
  n  = [1 n m];

  % reduce to unique points on the error curve
  [~,I,~] = unique([e1 e2],'rows');
  e1 = e1(I)';
  e2 = e2(I)';
  n  = n(I);

  % sort them horizontally, take care that identical values
  % are sorted vertically
  [~,R] = sort(e1-e2*1e-6);
  e1 = e1(R);
  e2 = e2(R);
  n  = n(R);
  if nargout <= 1 
    e.error = e1;
    e.xvalues = e2;
    e.xlabel = 'Error I';
    e.ylabel = 'Error II';
    e.title = [datname ' Clustering ROC'];
    e.plot = 'plot';
  end
  if nargout == 1
    e1 = e;
  end
  if nargout == 0
    clear e1
    plote(e);
  end
  
end