%CLUSTS Mean shift mode-seeking clustering
% 
% 	LAB = CLUSTS(A,K,MSIZE)
% 	LAB = A*CLUSTS(K,MSIZE)
% 
% INPUT
%   A    Dataset of M objects (rows).
%   K    Vector with desired numbers of clusters. Default is a set of N 
%        clusterings with numbers that naturally arise from the data.
% MSIZE  Number of objects (M) above which the dataset is preclustered by
%        CLUSTM, reducing it to MSIZE objects. Default MSIZE = 5000. Use
%        MSIZE = inf to avoid perclustering.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% This is a wrapper around MEANSHIFT, which is a wrapper around
% MeanShiftCluster routine by Bryan Feldman and Bart Finkston. These
% routines do not return clusterings with a preset number of clusters K.
% The resulting multilevel clustering is first made nested by RECLUSTN.
% After that the desired number of clusters is realized by RECLUSTK.
%
% EXAMPLE
% randreset;                     % take care of reproducability
% data = gendatclust1(20000);    % generate 20000 objects in 10 clusters
%                                % Run Mean Shift clustering
% lab = clusts(data,[2 5 10 18 30 50 100],2000);
%                                % Show scatterplot for 10 clusters
% figure; scatn(lab(:,3),data,'Mean Shift'); 
% figure; clusteval(lab,data);   % Evaluation by active learning
%
% REFERENCES
% Cheng, Y. "Mean shift, mode Seeking, and clustering", IEEE Transactions
% on PAMI, vol. 17, no. 8, pp. 790-799, 1995.
%
% R.P.W. Duin, A.L.N. Fred, M. Loog, and E. Pekalska, 
% Mode Seeking Clustering by KNN and Mean Shift Evaluated, 
% Proc. SSPR & SPR 2012, LNCS, vol. 7626, Springer, 2012, 51-59. 
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, CLUSTE, CLUSTF CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% RECLUSTN, PRECLUST, CLUSTEVAL, CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = clusts(varargin)

  argin = shiftargin(varargin,{'vector'},1);
  [a,K,msize] = setdefaults(argin,[],[],5000);
  if mapping_task(argin,'definition')
    labels = prmapping(mfilename,'fixed',{K,msize});
    labels = setname(labels,'MeanShift');
  else
    % execution
    a = +a;
    if size(a,1) >= msize
      % preclustering needed
      thisclust = feval(mfilename,[],K,msize);
      labels = preclust(a,thisclust,clustm,msize);
      if ~isempty(K)
        labels = reclustk(labels,K);
      end
    else
      labels = meanshift(a,[],false);
      if ~isempty(K)
        labels = reclustk(labels,K,labels(:,end-2));
      end
    end
  end
  
return


