%DCLUST2PROTO Find prototypes from given clustering of dissimilarity data
%
%   LAB_OUT = DCLUST2PROTO(D,LABC)
%   LAB_OUT = LABC*DCLUST2PROTO(D)
%
% INPUT
%   LABC      Index array, size [M,K], indices of cluster prototypes for M
%             objects in K clusterings.
%   D         M x M PRTools dissimilarity matrix or double array
%
% OUTPUT
%   LAB_OUT   Vector with set of M indices pointing for every object in A
%             to the centre of its cluster as derived from D.
%
% DESCRIPTION
% By this routine cluster labels of a dataset D are replaced by the indices
% of the centres of these clusters. A cluster centre is the object for
% which the maximum distance to the other objects in the cluster is minimized. 
%
% In case the cluster labels LAB_IN are not given D should be a PRTools
% dataset and its labels are interpreted as the cluster labels.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, EXEMPLAR, DCLUSTE, DCLUSTM, DCLUSTH, DCLUSTK, 

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com


function out = dclust2proto(varargin)

argin = shiftargin(varargin,'last',1);
argin = setdefaults(argin,[],[]);
if mapping_task(argin,'definition') % facilitates a*clus2proto for PRTools
  out = define_mapping(argin,'fixed');
else % normal call
  [lab,a] = deal(argin{:});
  if isempty(lab) % facilitates a*dclus2proto for PRTools
    if isa(a,'dataset')
      lab = getnlab(a);
    else
      error('No cluster labels supplied');
    end
  end
  a = +a;
  m = size(a,1);
  if size(lab,1) ~= m
    error('Wrong number of cluster labels supplied');
  end
  nc = size(lab,2);
  out = zeros(m,nc);
  t = sprintf('Processing %i nc clusterings: ',nc);
  prwaitbar(nc,t);
  for j=1:nc                        % run over all clusterings
    prwaitbar(nc,j,[t num2str(j)]);
    labj = lab(:,j);
    clusters = unique(labj);
    k = numel(clusters);
    for i=1:k                        % run over all clusters
      L = find(labj == clusters(i)); % objects in this cluster
      d = a(L,L);                    % distances in cluster
      [~,n] = min(max(d));           % centre object 
      out(L,j) = repmat(L(n),numel(L),1); % let all objects point to it
    end
  end
  prwaitbar(0);
end
      
    