%DCLUSTF Dissimilarity based clustering by the FFT algorithm.
% 
% 	LAB = DCLUSTF(D,K)
% 	LAB = D*DCLUSTF(K)
%
% INPUT
%  D     Square dissimilarity matrix, size M*M
%  K     Scalar or a vector of length N with desired numbers of clusters.
%        Default is a set of N clusterings with numbers that naturally
%        arise from the data.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% The FFT (Farthest First Traversal) algorithm is used to find a set of K
% prototypes. All other objects are assigned to the nearest prototype.
% In LAB for all objects the index in D of the nearest cluster prototype is
% returned. If K is a vector LAB has length(K) columns, returning a 
% multilevel clustering. 
%
% Note that the FFT algorithm is not really a clustering procedure but just
% a fast way to split the data in spatially distant parts.
%
% Clusterings can be evaluated by CLUSTEVAL, CLUSTCERR or CLUSTC on the
% basis of (some) true labels.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KMEANS, PRKMEANS, CLUSTK, DCLUSTM, DCLUSTH, DCLUSTE,
% CLUSTEVAL, CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = dclustf(varargin)

argin = setdefaults(varargin,[],[]);
if mapping_task(argin,'definition')
  labels = prmapping(mfilename,'fixed');
else
  [d,k] = deal(argin{:});
  m = size(d,1);
  n = numel(k);
  if isempty(k)
    % generate set of k-values using a geometric series
    k = intgeos(2,1.21,m/5);  % neighborhood sizes in kNN modeseeking
  end
  labels = zeros(m,n);
  P = dselproto(d,max(k),'fft');
  for i=1:n
    [~,r] = min(d(:,P(1:k(i))),[],2);
    labels(:,i) = P(r);
  end
end
return
