%DCLUSTH Dissimilarity based hierarchical clustering 
%
%   [LAB,DEN] = DCLUSTH(D,K,TYPE)
%   [LAB,DEN] = D*DCLUSTH([],K,TYPE)
%
% INPUT
%  D    Square dissimilarity matrix, size M*M
%  K    Vector of length N with desired numbers of clusters.
%       Default is a sampling of the [2:M] interval.
%  TYPE Character idicating linkage type:
%       'a' - average linkage
%       'c' - complete linkage
%       'r' - central linkage
%       's' - single linkage (default)
%  
% OUTPUT
%  LAB   [M,N] array with the results of the multilevel clusterings of the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%  DEN   Dendrogram, see PLOTDG or DENDROGRAM
%
% DESCRIPTION
% A hierarchical clustering is performed on the dissimilarity matrix D
% of M objects. The cluster indices for the N desired cluster levels stored
% in K are returned in LAB.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, CLUSTH, PLOTDG, DCLUSTM, DCLUSTK, DCLUSTE, CLUSTEVAL,
% CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [out,den] = dclusth(varargin)

  argin = shiftargin(varargin,'vector',1);
  argin = setdefaults(argin,[],[],'s');
  name = linktype(argin{3});
  if mapping_task(argin,'definition')
    out = define_mapping(argin,'fixed',name);
  else
    [d,k,type] = deal(argin{:});
    [~,type] = linktype(type);
    [m,mm] = size(d);
    if m~=mm
      error('Square dissimilarity matrix expected')
    end
    if isempty(k), k = intgeos(2,1.21,m/2); end
    if ~isa(k,'double')
      error('Second argument should be integer')
    end
    
    %lab = zeros(m,numel(k));
    den = hclust(+d,type,[],true);
    if nargout > 1
      [lab,den] = hclustdg(den,k);
      den = pr2mat_den(den);
    else
      lab = hclustdg(den,k);
    end

    if isdataset(d)
      lab = setdata(d,lab);
    end
    
    out = dclust2proto(lab,d);
    
  end
  
  %HCLUSTDG Find multi-clustering from a given dendrogram
%
%  [LABELS, DEN] = HCLUSTDG(DEN,K)
% 
% INPUT
%  DEN        PRTools dendrogram computed by HCLUST(D,TYPE,[],TRUE)
%  K          number of clusters, possibly a set of numbrs, default K = 2
%
% OUTPUT
%  LABELS     vector with labels
%  DEN        matrix with reduced PRTools dendrogram
%
% DESCRIPTION
% Computes a reduced dendrogram for plotting by PLOTDG as well as the
% corresponding cluster labels of the original objects for a desired number
% of clusters K. In case K is a set of cluster numbers, LABELS has multiple
% columns.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% HCLUST, PLOTDG, CLUSTH, DCLUSTH

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [labels,dendrogram] = hclustdg(den,k)

  if nargin < 2 || isempty(k), k = 2; end
  m = size(den,2);

  W = den(1,:);  % the object indices
  F = [den(2,:) inf];  % the cluster level

  kmax = max(k);
  knum = numel(k);
  labels = zeros(m,knum);
  lab = zeros(m,1);
  [~,J] = sort(-F);         % find cluster level
  k = sort(k);
  for j=1:knum
    I = sort(J(1:k(j)+1));  % find all indices where cluster starts
    for i=1:k(j)
      lab(W(I(i):I(i+1)-1)) = i * ones(I(i+1)-I(i),1);
    end
    labels(:,j) = lab;
  end
  dendrogram = [1:kmax; F(I(1:kmax))];
  %dendrogram = [I(2:kmax+1) - I(1:kmax); F(I(1:kmax))];
return
  
return
