%DCLUSTK Dissimilarity based clustering by k-centres or k-medoids
% 
%   LAB = DCLUSTK(D,K,TYPE,L)
%   LAB = D*DCLUSTK(K,TYPE,L)
% 
% INPUT
%  D    Square dissimilarity matrix, size M*M
%  K    Vector of length N with desired numbers of clusters, 
%       default is a sampling of [2:M]
%  TYPE 'kcentres', k-centres (default)
%       'kmedoids', k-medoids
%  L    Vector length max(K), indices of initial centres. Default:
%       initialisation by DSELPROTO.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION  
% Finds K prototypes from a symmetric distance matrix D. The prototypes 
% are chosen from all M objects such that the maximum (for k-centres) or
% mean (for k-medoids) of the distances over all objects to the nearest 
% prototype is minimized.
%
% In case K is a vector (length N), the procedure is repeated for as set of
% values for K between 2 and M.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KMEANS, PRKMEANS, CLUSTK, DCLUSTM, DCLUSTH, DCLUSTE,
% CLUSTEVAL, CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = dclustk(varargin)

  argin = shiftargin(varargin,'vector',1);
  argin = shiftargin(argin,'char',2);
  argin = setdefaults(argin,[],[],'kcentres',[],false);
  if mapping_task(argin,'definition')
    labels = define_mapping(argin,'fixed');
  else
    [d,k,type,N,checked] = deal(argin{:});
    type = clustertype(type); % make type either kcentres or kmedoids
    m = size(d,1);
    if ~checked        % avoid initialisation in case of recursive calls
      % check and normalize dissimilarity matrix
      %d = checkdissim(d);
      if isempty(k)
        k = intgeos(2,1.21,m/5);
      end
    end
    k = k(k<=m); % allow very small d
    numk = numel(k);
    if numk > 1
      labels = zeros(m,numk);
      t = sprintf([type ', %i clusterings: '],numk);
      prwaitbar(numk,t);
      for j=1:numk
        prwaitbar(numk,j,[t num2str(j)]);
        labels(:,j) = feval(mfilename,d,k(j),type,N,true);
      end
      prwaitbar(0);
      return
    end

    % this is where the work is done !!!
    if (k == 1) % Here we find the centre / medoid of all points stored in d
       % find for every potential prototype the maximum / mean distance
      switch type
        case 'kcentres'
          dm = max(d);
        case 'kmedoids'
          dm = mean(d);
      end
      [~,Jopt] = min(dm); % find for which one this is minimum
      labels = repmat(Jopt,m,1); % use it to label all objects
      return;
    end

    if k > m
      error('Number of desired prototypes should not exceed number of objects')
    end
    % We are here only if k (> 1) centers are to be found.
    if isempty(N)
      N = dselproto(d,k,type); % systematic initialisation
    end
    M = N(1:k);           % Take k initial prototypes
    J = zeros(1,k); 			% Center objects to be found.

    starttime = clock;
    runtime = 0;
    t = sprintf('Clustering, iteration: ');
    prwaitbar(prtime,t);

    % Iterate until J == M. See below.
    iter = 1;
    while 1,
      iter = iter+1;
      prwaitbar(prtime,runtime,[t num2str(iter)]);
      [dd,I] = min(d(:,M),[],2); % find for every object its nearest prototype
      % Find k centers. 
      for i = 1:k                   
        JJ = find(I==i); 
        if numel(JJ) > 1
          % Find objects closest to the object M(i)
          j = feval(mfilename,d(JJ,JJ),1,type,N,true); 
          % j is vector of indices of prototypes (identical as k=1)
          J(i) = JJ(j(1));
        else
          J(i) = JJ;
        end
      end

      J(J==0) = []; % cluster deleted, should not happen
      runtime = etime(clock,starttime);
      if ((length(M) == k) && (all(M(:) == J(:)))) || iter >= 10 || runtime > prtime
        % k centers are found.
        [~,labs] = min(d(:,J),[],2);
        break;
      end
      M = J;

    end
    if runtime > prtime
      prwarning(2,['DCLUSTK clustering updating stopped by PRTIME after ' num2str(iter) ' iterations']);
    end
    prwaitbar(0);

    %labels = labs';
    labels = J(labs)'; % return indices as labels

    if isdataset(d)
      labels = setdata(d,labels);
    end
  end
return
