%DCLUSTM Dissimilarity based multi-level clustering by kNN mode-seeking.
% 
% 	LAB = DCLUSTM(D,K)
% 	LAB = D*DCLUSTM(K)
%
% INPUT
%  D     Square dissimilarity matrix, size M*M
%  K     Scalar or a vector of length N with desired numbers of clusters.
%        Default is a set of N clusterings with numbers that naturally
%        arise from the data.
%
% OUTPUT
%  LAB   M*N array with the results of the multi-level clusterings for the
%        M objects. The columns refer to the N clusterings. They yield for
%        the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% A kNN modeseeking method is used to assign each object to its nearest
% density mode. Object densities are related to the distances to neighbors.
% Modes are determined by recusively jumping to objects in the neighborhood
% with the highest density. As many clusters are found as there are objects
% that are the mode in their own neighborhood. 
%
% Modeseeking clustering does not return a predefined number of clusters.
% Determining a clustering with exactly K clusters is done by RECLUSTK. It 
% might reduce the perfomance.
%
% This routine is based on the same algorithm as CLUSTM (for feature based
% data), which uses MODECLUST and MODECLUSTF. For reasons of speed it is
% implemented in another way, so results may differ slightly.
%
% Clusterings can be evaluated by CLUSTEVAL, CLUSTCERR or CLUSTC on the
% basis of (some) true labels.
% 
% REFERENCES
% Cheng, Y. "Mean shift, mode Seeking, and clustering", IEEE Transactions
% on PAMI, vol. 17, no. 8, pp. 790-799, 1995.
%
% R.P.W. Duin, A.L.N. Fred, M. Loog, and E. Pekalska, 
% Mode Seeking Clustering by KNN and Mean Shift Evaluated, 
% Proc. SSPR & SPR 2012, LNCS, vol. 7626, Springer, 2012, 51-59. 
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTK, DCLUSTH, DCLUSTE, MODECLUST, MODECLUSTF,
% KCLUST, RECLUSTN, CLUSTEVAL, CLUSTCERR, CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [labels,k] = dclustm(varargin)

argin = shiftargin(varargin,'vector',1);
argin = shiftargin(argin,'logical',2);
argin = setdefaults(argin,[],[],[]);
if mapping_task(argin,'definition')
  labels = define_mapping(argin,'fixed');
  labels = setname(labels,'ModeSeek');
else
  [a,c,nest] = deal(argin{:});
  m = size(a,1);
  % generate set of k-values using a geometric series
	k = intgeos(2,1.21,m/5);  % neighborhood sizes in kNN modeseeking
	[d,J] = sort(+a,2);		    % Sort and find neighbours.
	%f = 1./(d(:,k)+realmin); % Calculate densities for all values in k
	f = d(:,k); % Calculate densities for all values in k
	% run over all values in k
	nk = numel(k);
  labels = zeros(m,nk);
  
	for n = 1:nk % The main clustering loop
    
		% Find indices of local modes in neighbourhood.
		ff = f(:,n);
		[~,I] = min(reshape(ff(J(:,1:k(n))),m,k(n)),[],2);

		% Translate back to indices in all the data. N now contains the
		% index of the nearest neighbour in the K-neighbourhood.
		N = J([1:m]'+(I-1)*m);
		
		% Re-assign samples to the sample their nearest neighbour is assigned to.
		% Iterate until assignments don't change anymore. Samples that then point 
		% to themselves are modes; all other samples point to the closest mode.
		M = N(N);
    while (any(M~=N))
			N = M; M = N(N);
    end
    labels(:,n) = M;
		
  end 
  
  labels = labsort(labels,'downred');
  labels = fliplr(labels);
  if ~isempty(nest) && nest && ~isempty(c)
    prwarning(2,'As number of clusters is demanded, nesting had to be switched off')
    nest = false;
  end
  nest = setdefaults(nest,true);
  if nest
    labels = reclustn(labels,J(:,2));
  end
  if ~isempty(c)
    labels = reclustk(labels,c,labels(:,3));
  end
end

return
