%DSELPROTO Select prototypes from given dissimilarity matrix
%
%   N = DSELPROTO(D,K,CRIT,NINIT)
%   N = D*DSELPROTO(K,CRIT,NINIT)
%
% INPUT
%   D     Dissimilarity matrix 
%         between objects (rows) and protoypes (columns)
%   K     Number of prototypes to be selected
%   CRIT  'kmeans', 'kmedoids' or 'kcentres' (default: 'kmeans')
%   NINIT Indices of preselected prototypes (optional)
%
% OUTPUT
%   N     Indices of selected prototypes
%
% DESCRIPTION
% Select and rank a subset of K prototypes (columns) of the matrix D using
% a greedy forward selection approach such that the maximum NN distance 
% from all objects to these prototypes minimizes: 
% - for CRIT is 'kmeans' or 'kmedoids': mean(min(D(:,N(1:K),[],2)).
% - for CRIT is 'kcentres'            : max(min(D(:,N(1:K),[],2)).
%
% This routines tries to sample the given prototypes (possibly the original
% objects) such that they are evenly spaced judged from the dissimilarities. 
% This may be used as a systematic initialisation in kmeans, kmedoids or
% kcentres procedures. 
%
% The dissimilarity matrix D might be rectangular (prototypes different
% from objects). It might also be asymmetric.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, SELPROTO

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function N = dselproto(varargin)

  argin = shiftargin(varargin,'scalar',1);
  argin = setdefaults(argin,[],[],'kmeans',[]);
  if mapping_task(argin,'definition')
    % definition of the mapping in case of A*CLUSTK([],PARS)
    N = define_mapping(argin,'fixed');
  else
    % retrieve diss. matrix, requested number of protoypes and criterion
    [d,p,crit,Ninit] = deal(argin{:});
    d = +d; % get rid of PRTools dataset type
    if any(d(:)<0)
      prwarning(1,'Some values are negative. Is this a dissimilarity matrix?')
    end
    [nobj,nprot] = size(d);
    if isempty(p), p = nprot; end
    if p > nprot
      error('More prototypes requested than available')
    end
    crit = clustertype(crit);
    k0 = length(Ninit);
    if k0 > p
      prwarning(3,'More preselected prototypes given than requested number')
      N = Ninit(1:p);
      return
    end
    
    
    % Intitialisation
    L = true(1,nprot);      % Available set of candidate prototypes
    L(Ninit) = false;       % Remove preselected prototype
    N = zeros(p,1);         % Selected prototypes
    N(1:k0) = Ninit;        % Include preselected prototypes
    e = inf(nobj,1);        % Distance to nearest prototype (dNP)
    if k0 > 0
      e = min(d(:,N(1:k0)),[],2);
    end
    f = min(d,repmat(e,1,nprot)); % Replace larger distances by dNP

    % Find prototypes
    t = sprintf('Find %i prototypes: ',p);
    prwaitbar(p,t);
    for k=k0+1:p        
      prwaitbar(p,k,[t num2str(k)])
      if strcmp(crit,'kcentres')
        % Select the best prototype out of candidates in L 
        [~,n] = min(max(f(:,L)));       
      elseif strcmp(crit,'kmeans') || strcmp(crit,'kmedoids')
        % Select the best prototype out of candidates in L 
        [~,n] = min(mean(f(:,L)));
      elseif strcmp(crit,'fft')
        % Select the best prototype out of candidates in L 
        if k ==1
          [~,n] = max(sum(d(:,L)));
        else
          [~,n] = max(min(d(N(1:k-1),L),[],1),[],2);
        end
      else
        error(['Unknown criterion: ' crit])
      end
      R = find(L);
      e = min(d(:,R(n)),e);       % update dNP
      f = min(d,repmat(e,1,nprot)); % update large distance replacement
      N(k) = R(n);                % update list of selected prototypes
      L(R(n)) = false;            % update list of candidate prototypes
    end
    prwaitbar(0)
  end
  