%EXEMPLAR Examplar clustering
% 
% 	LAB = EXEMPLAR(D,P,F)
% 
% INPUT
%  D   Square dissimilarity matrix, size M*M
%  P   Vector (length N) with preferred self-distance >= 0
%  F   Damping factor, default 0.5
%
% OUTPUT
%  LAB  M*N array with in its columns the cluster indices for the M objects
%
% DESCRIPTION
%  This routine performs a clustering based on message passing between data
%  points, see [1]. The original code supplied by the authors is followed
%  with a few exceptions or extensions:
%  - the dissimilarity matrix D is automatically converted into a
%  similarity matrix with values between 0 (corresponding with the largest
%  dissimilarity) and 1 (corresponding with dissimilarities 0).
%  - the diagonal of the similarity matrix is replaced by a set of
%  identical values to one entry in P.
%  - the routine is run for all N elements of P, resulting in N
%  clusterings.
%  - the number of iterations is set to inf. The  iteration loop is broken
%  when 3 times in a row an identical clustering is found. Alternatively,
%  updating will be stopped by PRTIME. For larger datasets (M > 1000) and
%  large self-similarities P updating is time consuming and large values
%  of PRTIME might be needed (e.g. >500) to obtain useful results.
%  - the default value for the damping factor (averaging between two
%  successive iterations) has been set to 0.5 as proposed by the authors.
%  Much faster  results (less iterations) are obtained by higher damping
%  factors like 0.95.
% 
%  In case P is omitted an attempt is made to generate a set of possible
%  values such that a reasonable set of clusterings is obtained between 2
%  and M clusters.
% 
% REFERENCES
%   [1] B.J. Frey and D. Dueck, Clustering by passing messages between
%       data points, Science, vol. 315, pp. 972-976, 2007
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTM, DCLUSTH, ,DCLUSTK, CLUSTEVAL, CLUSTCERR,
% CLUSTC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com


function lab = exemplar(varargin)
	
  argin = setdefaults(varargin,[],[],0.5,false);
  [d,alf,lam,checked] = deal(argin{:});
  if ~checked  % avoid initialisation in case of recursive calls
    % check and normalize dissimilarity matrix
    d = drankm(d);
    d = checkdissim(d);
  end
  %m = size(d,1);

  global TOOLONG
  if isempty(alf)
    %numalf = 20;
    numalf = 50;
    alf = zeros(1,numalf);
    alf(1) = 0.01;
    defalf = true;
    for j=2:numalf
      %alf(j) = alf(j-1)*1.6;
      alf(j) = alf(j-1)*1.6;
    end
    %alf = alf+0.003;
  else
    defalf = false;
  end
  numalf = numel(alf);
  if numalf > 1
    alf = sort(alf);
    lab = zeros(size(d,1),numalf);
    t = sprintf('Examplars, %i clusterings: ',numalf);
    prwaitbar(numalf,t);
    TOOLONG= false;
    for j=1:length(alf)
      randreset;
      prwaitbar(numalf,j,[t num2str(j)]);
      lab(:,j) = feval(mfilename,d,alf(j),lam,true);
      %disp([j,alf(j),numel(unique(lab(:,j)))]);
      if defalf && j > 5 && numel(unique(lab(:,j))) < 3 || TOOLONG
        % better to stop if TOOLONG as larger alf will also be stopped
        % prematurely
        break;
      end
    end
    prwaitbar(0)
    if defalf
      L = (max(lab,[],1) ~= 0);
      lab = labsort(lab(:,L),'upred');
    end
    return
  end

  S = 1 - d;
  S = S-alf*eye(size(S)); % the larger alfa, the fewer clusters

  % the below lines for itermax = 100 are exactly the code given in
  % the supporting material of [1]

  N=size(S,1); A=zeros(N,N); R=zeros(N,N); % Initialize messages
  S=S+1e-12*randn(N,N)*(max(S(:))-min(S(:))); % Remove degeneracies
  %lam=0.5; % Set damping factor
  %lam=0.95;

  starttime = clock;
  runtime   = 0;
  iter      = 0;
  t = sprintf('Clustering, iteration: ');
  prwaitbar(prtime,t);

  Lold  = zeros(N,1);
  Lold2 = ones(N,1);
  while 1
    iter = iter+1;
    prwaitbar(prtime,runtime,[t num2str(iter)]);
    % Compute responsibilities
    Rold=R;
    AS=A+S; [Y,I]=max(AS,[],2);
    for i=1:N, AS(i,I(i))=-realmax; end;
    Y2=max(AS,[],2);
    R=S-repmat(Y,[1,N]);
    for i=1:N, R(i,I(i))=S(i,I(i))-Y2(i); end;
    R=(1-lam)*R+lam*Rold; % Dampen responsibilities
    % Compute availabilities
    Aold=A;
    Rp=max(R,0); 
    for k=1:N, Rp(k,k)=R(k,k); end;
    A=repmat(sum(Rp,1),[N,1])-Rp;
    dA=diag(A); A=min(A,0); 
    for k=1:N, A(k,k)=dA(k); end;
    A=(1-lam)*A+lam*Aold; % Dampen availabilities

    L = diag(R+A) > 0;
    runtime = etime(clock,starttime);
    if (sum(L) > 0 && sum(L) < N && all(L==Lold) && all(L==Lold2)) || ...
      runtime > prtime
      break;
    end
    Lold2 = Lold;
    Lold = L;
    %disp([alf,iter,sum(L)])
  end;
  if runtime > prtime
    prwarning(2,['Examplar clustering updating stopped by PRTIME after ' num2str(iter) ' iterations']);
    TOOLONG = true;  
  end
  prwaitbar(0);
  E=R+A; % Pseudomarginals
  I=find(diag(E)>0); K=length(I); % Indices of exemplars
  [~,c]=max(S(:,I),[],2); c(I)=1:K; idx=I(c); % Assignments

  % So far the code in [1]

  if isempty(idx) || numel(unique(idx)) == N
    lab = zeros(N,1);
  else
    lab = idx;
  end

return