%LABNN Find indices of nearest neighbors
%
%   LABELS = LABNN(TEST,TRAIN,K,PROC)
%   LABELS = TEST*LABNN([],TRAIN,K,PROC) for batch processing
%   LABELS = TEST*LABNN                  for finding nearest neighbors
%
% INPUT
%   TEST  - Dataset with M objects, doubles or PRTools dataset
%   TRAIN - Dataset with N objects, doubles or PRTools dataset
%   K     - Neighbor number to be found
%   PROC  - String indicating procedure to be used in the search, see below.
%
% OUTPUT
%   LABELS - Array with size MxK of indices pointing for each element of
%            TEST to the K nearest neighbors in TRAIN.
%
% DESCRIPTION
% In PROC one of the following routines can be defined:
%  - 'prtools'  exact computation using PRTools (default)
%  - 'ann'      approximate computation, needs the ANN Matlab Wrapper
%  - 'kdtree'   using Matlab's KNNSEARCH
%
% These procedures are evaluated in an <a href="http://37steps.com/exam/fast_knn_comparison.txt">extensive experiment</a>.
% The <a href="http://ann-matlab-wrapper.script.soft32download.com/">ANN Matlab Wrapper package</a> should be in the path.
% Any of these three method can be considerably faster than the other two
% for some sizes of TRAIN, TEST.
%
% A special case is defined by TRAIN = [], which results LABELS to point to
% the nearest neighbors of TEST to itself. This is defined only for PROC
% is 'prtools'.
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, PRMEM, PROXM, KNNSEARCH

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labels = labnn(varargin)

  argin = setdefaults(varargin,[],[],1,'prtools');
  if mapping_task(argin,'definition')
    labels = define_mapping(argin,'fixed');
  else
    [a,trainset,k,proc] = deal(argin{:});
    if isempty(trainset) 
      loo = true;
      trainset = a;
    else
      loo = false;
    end

    proc = lower(proc);

    if ismapping(trainset) % possible knnc
      trainset = getdata(trainset,1);
    end

    switch proc
      case 'prtools'
        m = size(a,1);
        n = size(trainset,1);
        compnn = proxm(trainset,[],2);
        [loops,rows,last] = prmem(m,n);
        if loops == 1
          labels = getlabnn(a,compnn,k,loo);
        else
          labels = zeros(m,max(k,1));
          t = sprintf('Finding %i neighbors: ',m);
          prwaitbar(m,t);
          for j=1:loops
            if j==loops, nrows = last; else, nrows = rows; end
            nn = (j-1)*rows;
            prwaitbar(m,nn+nrows,[t num2str(nn+nrows)]);
            labels(nn+1:nn+nrows,:) = getlabnn(a(nn+1:nn+nrows,:),compnn,k,loo);
          end
          prwaitbar(0);
        end
      case 'ann'
        labels = annquery(+trainset',+a',k)';
  %     case 'fast_knn'
  %       labels = fast_knn(+a',+trainset',k)';
      case 'kdtree'
        labels = knnsearch(+trainset,+a,'K',k,'NSMethod','kdtree');
      otherwise
        error('NN procedure not found')
    end
  end
return
  
function labels = getlabnn(a,compnn,k,loo)
  d = +a*compnn;
  if loo
    d = d+diag(inf(1,size(d,1)));
  end
  [~,L] = sort(d,2);
  labels = L(:,1:k);
return