%LABSORT Sort label array of a multiclustering
%
%   [LABOUT,IND] = LABSORT(LABIN,TYPE,MAXSIZE)
%   [LABOUT,IND] = LABIN*LABSORT(TYPE,MAXSIZE)
%
% INPUT
%   LABIN  Numeric label array, size MxL, M objects, L levels.
%   TYPE   'up' sort columns according to increasing number of clusters.
%          'down' sort columns according to decreasing number of clusters.
%          'upred' sort columns according to increasing number of 
%          clusters and remove columns with equal numbers.
%          'downred' sort columns according to increasing number of 
%          clusters and remove columns with equal numbers.
%          Default: 'up
%
% OUTPUT
%  LABOUT  MxN array with the results of the multilevel clusterings of the
%          M objects. The sorted columns refer to N clusterings in an order
%          as defined by TYPE.
%   IND    Sorting index used, LABOUT = LABIN(:,IND)
%
% DESCRIPTION
% This is a support routine used by several procedures that analyse the
% output of a multiclustering. The rows of LABIN are ranked according to
% the numbers of clusters. In addition rows pointing to clusterings with
% the same number of clusters are reduced.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTH, CLUSTE, CLUSTF CLUSTK, CLUSTH, CLUSTKH, 
% CLUSTM, RECLUSTK, RECLUSTN, RECLUSTH, CLUSTEVAL, CLUSTCERR, CLUSTC,
% CLUSTNUM, PLOTDG

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [labout,L] = labsort(varargin)

  argin = shiftargin(varargin,'char',1);
  argin = setdefaults(argin,[],'up',[]);
  if mapping_task(argin,'definition')
    labout = define_mapping(argin,'fixed');
  else
    [labin,type,maxsize] = deal(argin{:});
    N = clustnum(labin);
    switch lower(type)
      case 'up'
        [~,L] = sort(N);
      case 'down'
        [~,L] = sort(N,'descend');
      case 'upred'
        [~,L] = unique(N);
      case 'downred'
        [~,ir] = unique(N);
        L = flipud(ir);
      otherwise
        error('Unknown type')
    end
    labout = labin(:,L);
    if ~isempty(maxsize)
      L = clustnum(labout);
      labout = labout(:,L<=maxsize);
    end
  end
    
    