%MCLUSTCERR Cluster classification error based on combining clusterings
%
%   [E,N} = MCLUSTCERR(LABC,LABT)
%   [E,N] = MCLUSTCERR(LABC,A)
%   [E,N] = LABC*MCLUSTCERR(LABT)
%
% INPUT
%   LABC   Index array, size [M,K], indices of cluster prototypes for M
%          objects in K clusterings.
%   LABT   Vector of M elements with true object labels.
%   A      PRTools labeled dataset used for the clustering.
%
% OUTPUT
%   E      Vector with classification errors of the clusterings in LABC.
%   N      Vector with number of clusters per clustering.
%
% DESCRIPTION
% This routine evaluates a clustering of a dataset A by comparing the true
% labels LABT of A with cluster labels derived from a cluster prototype
% object by combining the multilevel clustering LABC using CLUSTC. This
% can be understood as evaluating the clusterings by active labeling.
%
% A is a labeled dataset with M objects. LABT should be a vertical vector 
% containing the true numeric labels of A. LABT = GETNLAB(A). LABC is a set
% of K clustering results with indices pointing for every object to a 
% cluster prototype in A.
%
% E is the fraction of misclassified objects in A. It reports an error for
% every clustering (column of LABC). Use CLUSTCERR for a faster result that
% does not combine cluster levels and which has thereby usually a smaller
% performance. The length of E and N might differ from K.
%
% EXAMPLE
% A = gendat(mnist8,25000);
% labc = A*clustm([],false); % do not recluster
% [e1,n1] = clustcerr(labc,A);
% [e2,n2] = mclustcerr(labc,A);
% figure;
% semilogx(n1,e1); hold on
% semilogx(n2,e2);
% title(['Active learning curve: ' getname(A)])
% xlabel('Training set size - number of clusters')
% ylabel('Classification error')
% legend('Single clusterings','Combined clusterings')
% fontsize(16)
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, CLUSTR, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR,
% CLUSTEVAL, CLUSTC, CLUSTNUM, CLUSTCERR, MCLUSTLCURVE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [e,n] = mclustcerr(varargin)

  argin = shiftargin(varargin,'last',1);
  argin = setdefaults(argin,[],[]);
  if mapping_task(argin,'definition')
    e = define_mapping(argin,'fixed');
  else
    [labc,labt] = deal(argin{:});
    labt = truelabs(labt);
    labc  = labsort(labc,'downred');
    nclust = size(labc,2);
    e = zeros(1,nclust);
    n = zeros(1,nclust);
    t = sprintf('Analysing %i clusterings ',nclust);
    prwaitbar(nclust,t)
    for k = 1:nclust
      prwaitbar(nclust,k,[t num2str(k)]);
      labtt = zeros(size(labt,1),1);
      N = unique(labc(:,k)); % prototypes
      labtt(N) = labt(N);    % true labels for prototypes only
      if k==1                % no combining for single level clustering
        e(k) = mean(labt ~= labt(labc(:,k)));
      else
        labe = clustc(labc(:,1:k),labtt);
        e(k) = mean(labe~=labt);
      end
      n(k) = numel(N);
    end
    prwaitbar(0)
    n = fliplr(n);
    e = fliplr(e);
  end