%MCLUSTLCURVE Compute learning from multilevel clustering classification
%
%  E = MCLUSTLCURVE(LABC,LABT)
%  E = MCLUSTLCURVE(LABC,A)
%  E = LABC*MCLUSTLCURVE(LABT)
%  E = LABC*MCLUSTLCURVE(A)
%
% INPUT
%   LABC   Index array, size [M,K], indices of cluster prototypes for M
%          objects in K clusterings.
%   LABT   Vector of M elements with true object labels.
%   A      PRTools labeled dataset used for the clustering.
%
% OUTPUT
%   E      Structure with classification error and numbers of labeled
%          objects, ready to plot by PLOTE
%
% DESCRIPTION
% This routines calls MCLUSTCERR to create a learning curve ready to be 
% plot by PLOTE. This learning curve evaluates active learning in which
% objects are assigned to the class of cluster prototypes after combining
% the multilevel clustering.
%
% If no output (E) is requested CLUSTLCURVE calls PLOTE internally and
% thereby produces automatically a figure with a learning curve.
%
% EXAMPLE
% dset = gendatm(30000);
% labc = dset*clustm;
% mclustlcurve(labc,dset)
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% CLUSTF, CLUSTR, DCLUSTE, DCLUSTF, DCLUSTH, DCLUSTK, DCLUSTM, DCLUSTR,
% CLUSTEVAL, CLUSTC, CLUSTNUM, MCLUSTCERR, PLOTE

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function e = mclustlcurve(varargin)

  argin = shiftargin(varargin,'last',1);
  argin = setdefaults(argin,[],[]);
  if mapping_task(argin,'definition')
    e = define_mapping(argin,'fixed');
  else
    [labe,labt] = deal(argin{:});
    labe = labsort(labe,'upred');
    [labt,name] = truelabs(labt);
    [err,sizes] = mclustcerr(labe,labt);
    e.error = err;
    e.xvalues = sizes;
    e.title    = ['Active learning curve ' name];
    e.xlabel   = 'Number of clusters';
    e.ylabel   = 'Classification error';
    e.plot     = 'plot';
    if max(sizes)/min(sizes) >= 100
      e.plot = 'semilogx';
    end
    if nargout == 0
      plote(e);
    end
  end

return