%MEANSHIFT Mean shift mode-seeking clustering
% 
% 	LAB = MEANSHIFT(A,K,NEST)
% 	LAB = A*MEANSHIFT(K,NEST)
% 
% INPUT
%   A     Dataset of M objects (rows).
%   K     Scalar or a vector of length N with neighbor numbers. Default is
%         a geometric series between 1 and M.
%   NEST  Logical, if TRUE (default) the output set of clusterings (columns
%         of LAB) is made nested by RECLUSTN.
%
% OUTPUT
%  LAB    M*N array with the results of the multi-level clusterings for the
%         M objects. The columns refer to the N clusterings. They yield for
%         the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% This is a wrapper around the MeanShiftCluster routine by Bryan Feldman
% and Bart Finkston found in <a href="http://nl.mathworks.com/matlabcentral/fileexchange/10161-mean-shift-clustering/">MathWork File Exchange</a>.
% The width parameter is estimated by the average K-NN distance. K can be a
% set of values, resulting in a multilevel clustering, stored in columns of
% LAB. Each index in LAB points the object of A that is most close to the
% corresponding mode of the cluster found by MeanShiftCluster.
%
% The multilevel clustering can be made nested by RECLUSTN.
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, DATASETS, KMEANS, HCLUST, MODECLUST, MODECLUSTF, DCLUSTM, 
% RECLUSTN

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [labels,p] = meanshift(varargin)

  argin = shiftargin(varargin,{'vector','logical'},1);
  argin = shiftargin(argin,'logical',2);
  argin = setdefaults(argin,[],[],true);
  if mapping_task(argin,'definition')
    % definition of the mapping
    labels = define_mapping(argin,'fixed');
  else
    [a,K,nest] = deal(argin{:});
    % prepare data
    a = +a;                 % get rid of possible PRTools dataset structure
    m = size(a,1);          % number of objects
    if isempty(K)
      K = intgeos(2,1.21,m/10); % default set of nb-sizes (neighborhood-sizes)
    end
    nk = numel(K);          % number of neighborhood sizes (clusterings)

    % estimate width parameters h from K
    if m <= 250
      d = sqrt(distm(a));
    else
      R = randperm(m); R = R(1:250);
      d = sqrt(distm(a(R,:),a));
    end
    dd = sort(d,2);
    h = mean(dd(:,K));
    
    % call MeanShiftCluster for all width parameters
    labels = zeros(m,nk);
    p = cell(1,nk);
    t = sprintf('Running over %i kernel sizes: ',nk);
    prwaitbar(nk,t)
    for j=1:nk
      prwaitbar(nk,j,[t num2str(j)]);
      [pj,labj] = MeanShiftCluster(+a',h(j));
      if isempty(pj) % clustering failed
        pj = +a';    % give every point its own cluster
        labels(:,j) = [1:m]';
      else
        L = labnn(pj',+a);
        labels(:,j) = L(labj');
      end
      p{j} = pj';
    end
    prwaitbar(0)
    labels = clust2proto(labels,a);
    if nest
    	labels = reclustn(labels);      % make set of clusterings nested
    else
      labels = fliplr(labels);
    end
  end
  
return

function [clustCent,data2cluster,cluster2dataCell] = MeanShiftCluster(dataPts,bandWidth,plotFlag);
%perform MeanShift Clustering of data using a flat kernel
%
% ---INPUT---
% dataPts           - input data, (numDim x numPts)
% bandWidth         - is bandwidth parameter (scalar)
% plotFlag          - display output if 2 or 3 D    (logical)
% ---OUTPUT---
% clustCent         - is locations of cluster centers (numDim x numClust)
% data2cluster      - for every data point which cluster it belongs to (numPts)
% cluster2dataCell  - for every cluster which points are in it (numClust)
% 
% Bryan Feldman 02/24/06
% MeanShift first appears in
% K. Funkunaga and L.D. Hosteler, "The Estimation of the Gradient of a
% Density Function, with Applications in Pattern Recognition"


%*** Check input ****
if nargin < 2
    error('no bandwidth specified')
end

if nargin < 3
    plotFlag = true;
    plotFlag = false;
end

%**** Initialize stuff ***
[numDim,numPts] = size(dataPts);
numClust        = 0;
bandSq          = bandWidth^2;
initPtInds      = 1:numPts;
maxPos          = max(dataPts,[],2);                    %biggest size in each dimension
minPos          = min(dataPts,[],2);                    %smallest size in each dimension
boundBox        = maxPos-minPos;                        %bounding box size
sizeSpace       = norm(boundBox);                       %indicator of size of data space
stopThresh      = 1e-3*bandWidth;                       %when mean has converged
clustCent       = [];                                   %center of clust
beenVisitedFlag = zeros(1,numPts,'uint8');              %track if a points been seen already
numInitPts      = numPts;                               %number of points to possibly use as initilization points
clusterVotes    = zeros(1,numPts,'uint16');             %used to resolve conflicts on cluster membership

t = sprintf('Restarting from %i points: ',numPts);
prwaitbar(numPts,t); nobj = 0;
while numInitPts
    nobj = nobj+1;
    if nobj == 100 % avoid large overhead by prwaitbar
      prwaitbar(numPts,numPts-numInitPts+1,[t num2str(numPts-numInitPts+1)]);
      nobj = 0;
    end

    tempInd         = ceil( (numInitPts-1e-6)*rand);        %pick a random seed point
    stInd           = initPtInds(tempInd);                  %use this point as start of mean
    myMean          = dataPts(:,stInd);                     % intilize mean to this points location
    myMembers       = [];                                   % points that will get added to this cluster                          
    thisClusterVotes    = zeros(1,numPts,'uint16');         %used to resolve conflicts on cluster membership

    starttime = clock;
    runtime = 0;
    while 1 && runtime < prtime    %loop untill convergence 
                                   % but stop if it takes too long (RD)
        sqDistToAll = sum((repmat(myMean,1,numPts) - dataPts).^2);    %dist squared from mean to all points still active
        inInds      = find(sqDistToAll < bandSq);               %points within bandWidth
        if isempty(inInds)
          clustCent = [];
          data2cluster = [];
          return
        end
        thisClusterVotes(inInds) = thisClusterVotes(inInds)+1;  %add a vote for all the in points belonging to this cluster
        
        
        myOldMean   = myMean;                                   %save the old mean
        myMean      = mean(dataPts(:,inInds),2);                %compute the new mean
        myMembers   = [myMembers inInds];                       %add any point within bandWidth to the cluster
        beenVisitedFlag(myMembers) = 1;                         %mark that these points have been visited
        
        %*** plot stuff ****
        if plotFlag
            figure(12345),clf,hold on
            if numDim == 2
                plot(dataPts(1,:),dataPts(2,:),'.')
                plot(dataPts(1,myMembers),dataPts(2,myMembers),'ys')
                plot(myMean(1),myMean(2),'go')
                plot(myOldMean(1),myOldMean(2),'rd')
                pause
            end
        end

        %**** if mean doesn't move much stop this cluster ***
        if norm(myMean-myOldMean) < stopThresh
            
            %check for merge posibilities
            mergeWith = 0;
            for cN = 1:numClust
                distToOther = norm(myMean-clustCent(:,cN));     %distance from posible new clust max to old clust max
                if distToOther < bandWidth/2                    %if its within bandwidth/2 merge new and old
                    mergeWith = cN;
                    break;
                end
            end
                 
            if mergeWith > 0    % something to merge
                clustCent(:,mergeWith)       = 0.5*(myMean+clustCent(:,mergeWith));             %record the max as the mean of the two merged (I know biased twoards new ones)
                %clustMembsCell{mergeWith}    = unique([clustMembsCell{mergeWith} myMembers]);   %record which points inside 
                clusterVotes(mergeWith,:)    = clusterVotes(mergeWith,:) + thisClusterVotes;    %add these votes to the merged cluster
            else    %its a new cluster
                numClust                    = numClust+1;                   %increment clusters
                clustCent(:,numClust)       = myMean;                       %record the mean  
                %clustMembsCell{numClust}    = myMembers;                    %store my members
                clusterVotes(numClust,:)    = thisClusterVotes;
            end

            break;
        end
        runtime = etime(clock,starttime);        
    end
    if runtime > prtime
      prwarning(2,['MeanShift clustering loop stopped by PRTIME']);
    end
    
    
    initPtInds      = find(beenVisitedFlag == 0);           %we can initialize with any of the points not yet visited
    numInitPts      = length(initPtInds);                   %number of active points in set
    
end
prwaitbar(0)

[val,data2cluster] = max(clusterVotes,[],1);                %a point belongs to the cluster with the most votes

%*** If they want the cluster2data cell find it for them
if nargout > 2
    cluster2dataCell = cell(numClust,1);
    for cN = 1:numClust
        myMembers = find(data2cluster == cN);
        cluster2dataCell{cN} = myMembers;
    end
end



