%MODECLUST KNN mode-seeking clustering
% 
% 	[LAB,NNLAB,NDIST] = MODECLUST(A,K,DIST,NEST)
% 
% INPUT
%   A       Dataset of M objects
%   K       Vector with numbers of neighbours to search for local mode.
%           Default: smart sampling.
%   DIST    Distance function (name or handle) or mapping to be used
%           for clustering (optional; default: @DISTM).
%           If DIST is a function it is expected to take two double arrays
%           as input arguments: if D = DIST(A1, A2) then SIZE(D) is
%           [SIZE(A1, 1) SIZE(A2, 1)]. If DIST is a mapping then it should
%           be possible to use it like D = A1*(A2*DIST), i.e. it has to be
%           a trainable mapping and it should be able automatically convert
%           double arrays to datasets. E.g., for PROXM it is possible to 
%           call MODECLUST(A,K,PROXM), but for DISTM we need to use
%           MODECLUST(A,K,@DISTM) or MODECLUST(A,K,'distm').
%   NEST    Logical, if TRUE the output set of clusterings (columns of LAB)
%           is made nested by RECLUSTN. Default: FALSE
%           
% OUTPUT
%   LAB     Indices of mode samples, size [M,N] with K the number of
%           clusterings (NUMEL(K)).
%   NNLAB   [M,1} vector with indices of nearest neighbors.
%   NDIST   Total number of distance calculations.
%
% DESCRIPTION
% A NN modeseeking method is used to assign objects to their nearest mode.
% Object densities are defined by one over the distance to the K-th nearest 
% neighbour. Clusters are defined by recursively jumping for every object 
% to the object with the highest density in the local neighborhood.
%
% K can be a vector of neighborhood sizes, which is much faster.
% Default K: a set of values determined by a geometric series.
%
% Computing time is in the order of M^2 seconds, with M the number of
% objects in A times 1000.
%
% REFERENCE
% R.P.W. Duin, A.L.N. Fred, M. Loog, and E. Pekalska, 
% Mode Seeking Clustering by KNN and Mean Shift Evaluated, 
% Proc. SSPR & SPR 2012, LNCS, vol. 7626, Springer, 2012, 51-59. 
%
% EXAMPLE
% delfigs
% a = gendatm(5000);      % generate 5000 objects in 8 classes
% [lab,k] = modeclust(a); 
% for j=1:size(lab,2)
%   nclust = numel(unique(lab(:,j)));
%   if nclust < 20 & nclust > 1
%     figure; scattern(prdataset(a,lab(:,j)));
%     title(['K = ' int2str(k(j)) ' --> ' int2str(nclust) ' Clusters']);
%   end
% end
% showfigs
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, DATASETS, DISTM, PROXM, CLUSTM, DCLUSTM, MODECLUST_BATCH, 
% MODECLUSTF, RECLUSTN

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com
% Option for arbitrary distances by Sergey Verzakov, s.verzakov@gmail.com


function [labels,T,K,ndist] = modeclust(varargin)

  argin = setdefaults(varargin,[],[],[],false);
  if mapping_task(argin,'definition')
    % definition of the mapping
    labels = define_mapping(argin,'fixed');
  else
    [a,K,dist,nest] = deal(argin{:});
    % prepare data
    a = +a;                 % get rid of possible PRTools dataset structure
    m = size(a,1);          % number of objects
    ndist = 0;              % track number of distance computations
    if isempty(dist)
      dist = @distm; % default distance 
    end

    % we will call feval(dist, a(idx,:), A)
    % to compute distances
    switch class(dist)
      case {'char', 'function_handle'}
        % A is just data
        A = a;
      case {'prmapping'}
        % A is trained mapping, dist is the application
        % of the mapping and convertin dataset to double
        A = a*dist;
        dist = @(x, y) +prmap(x, y);
      otherwise
        error('Wrong type of the DIST argument');
    end
    
    if size(a,1)^2 < prmemory && isempty(K) % distance matrix fits, so dclustm is faster
      d = feval(dist,a,A);
      labels = dclustm(d,[],nest);
      [~,J] = sort(+d,2);
      T = J(:,2);
      ndist = ndist + m*m;
      return
    end


    if isempty(K)
      K = intgeos(3,1.21,m/10); % default set of nb-sizes (neighborhood-sizes)
    end
    nk = numel(K);          % number of neighborhood sizes (clusterings)
    f = zeros(m,nk);        % space for storing densities
    batchsize = 25;         % batch sizes (avoid large distance matrices)
    nbatch = ceil(m/batchsize); % number of batches

    % first loop over the data: compute densities for every nb-size
    t = sprintf('Computing %i densities: ',m);
    prwaitbar(2*m+nk,t);
    for j=1:nbatch
      batch = (j-1)*batchsize+1:j*batchsize;  % take a subset of the objects
      if any(batch>m)              % arrange last batch
        batch = batch(batch <= m);
      end
      prwaitbar(2*m+nk,max(batch),[t num2str(max(batch))]);
      dj = feval(dist,a(batch,:),A); % distances between batch and all data
      dd = sort(dj,2);               % sort them per object
      f(batch,:) = 1./dd(:,K);       % density estimates for all nb-sizes
    end
    ndist = ndist+m*m;

    % second loop: find indices of maximum density point in every nb
    NN = zeros(m,nk);              % space for storing these indices
    T  = zeros(m,1);               % space for first neighbors
    t = sprintf('Checking %i neighborhoods: ',m);
    for j=1:nbatch
      batch = (j-1)*batchsize+1:j*batchsize;   % take a subset of the objects
      if any(batch>m)              % arrange last batch
        batch = batch(batch <= m)';
      end
      prwaitbar(2*m+nk,m+max(batch),[t num2str(max(batch))]);
      mj = numel(batch);             % number of data points in batch
      dj = feval(dist,a(batch,:),A); % recomputes distances
      [~,J] = sort(dj,2);           % sort them per object
      for n=1:nk                     % run over all nb's
        ff = f(:,n);                 % densities for this nb-size
        [~,I] = max(reshape(ff(J(:,1:K(n))),mj,K(n)),[],2);
        NN(batch,n)  = J((1:mj)'+(I-1)*mj); % for all points the index of
      end                          %  point with maximum local density 
      T(batch,1) = J(:,2);
    end
    ndist = ndist+m*m;

    % third loop: follow gradient to density modes
    labels = zeros(m,nk);
    t = sprintf('Computing %i clusterings: ',nk);
    for n=1:nk
      prwaitbar(2*m+nk,2*m+n,[t num2str(n)]);
      N = NN(:,n);                 % initial nb points with maximum density
      % Re-assign samples to the sample their nearest neighbour is assigned to.
      % Iterate until assignments don't change anymore. Samples that then point 
      % to themselves are modes; all other samples point to the closest mode.
      M = N(N);
      while (any(M~=N))
        N = M; M = N(N);
      end
      labels(:,n) = M;  % labels are the indices of the modes to which data 
                        % points are assigned to. 
      % Data points assigned to the same mode constitute a cluster
    end
    prwaitbar(0);
    if nest
      labels = reclustn(labels); 
    else
      labels = labsort(labels,'up');
    end
  end

return
