%MODECLUST_BATCH KNN mode-seeking clustering, batch processing for large datasets
% 
% 	[LAB, K] = MODECLUST_BATCH(A, K, DIST)
% 
% INPUT
%   A       Dataset
%   K       Vector with numbers of neighbours to search for local mode.
%           Default: smart sampling.
%   DIST    Distance function (name or handle) or mapping to be used
%           for clustering (optional; default: @DISTM).
%           If DIST is a function it is expected to take two double arrays
%           as input arguments: if D = DIST(A1, A2) then SIZE(D) is
%           [SIZE(A1, 1) SIZE(A2, 1)]. If DIST is a mapping then it should
%           be possible to use it like D = A1*(A2*DIST), i.e. it has to be
%           a trainable mapping and it should be able automatically convert
%           double arrays to datasets. E.g., for PROXM it is possible to 
%           call MODECLUST(A, K, PROXM()), but for DISTM we need to use
%           MODECLUST(A, K, @DISTM) or MODECLUST(A, K, 'distm').
%           
% OUTPUT
%   LAB     Indices of mode samples
%   K       The used input K vector (useful if K was not specified in the
%           call and was autimatically generated)
%
% DESCRIPTION
% A K-NN modeseeking method is used to assign objects to their nearest mode.
% Object densities are defined by one over the distance to the K-th nearest 
% neighbour. Clusters are defined by recursively jumping for every object 
% to the object with the highest density in the local neighborhood.
%
% K can be a vector of neighborhood sizes, which is much faster.
% Default K: a set of values determined by a geometric series.
%
% This routine is about twice as fast as fast as MODECLUST as by storing
% distances that are needed on disk.
%
% The multilevel clustering can be made nested by RECLUSTN.
%
% REFERENCE
% R.P.W. Duin, A.L.N. Fred, M. Loog, and E. Pekalska, 
% Mode Seeking Clustering by KNN and Mean Shift Evaluated, 
% Proc. SSPR & SPR 2012, LNCS, vol. 7626, Springer, 2012, 51-59. 
%
% EXAMPLE
% delfigs
% a = gendatm(5000);      % generate 5000 objects in 8 classes
% [lab,k] = modeclust(a); 
% for j=1:size(lab,2)
%   nclust = numel(unique(lab(:,j)));
%   if nclust < 20 & nclust > 1
%     figure; scattern(prdataset(a,lab(:,j)));
%     title(['K = ' int2str(k(j)) ' --> ' int2str(nclust) ' Clusters']);
%   end
% end
% showfigs
%
% 
% lab = modeclust_batch(a, [], @distm); % explicitly using DISTM
% lab = modeclust_batch(a, [], proxm([], 'm', 1)); % using PROXM mapping
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, DATASETS, DISTM, PROXM, DCLUSTM, MODECLUST_BATCH, MODECLUSTF.
% RECLUSTN

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com
% Option for arbitrary distances by Sergey Verzakov, s.verzakov@gmail.com


function [labels, K] = modeclust_batch(a,K,dist)

  % prepare data
  %a = +a;                 % get rid of possible PRTools dataset structure
  m = size(a,1);          % number of objects
  if (nargin < 3  || isempty(dist))
    dist = @distm; % default distance 
  end
  
  mkdir('fbatch');
  mkdir('ndens');
  mkdir('npoint');
  
  % we will call feval(dist, a(idx,:), A)
  % to compute distances
  switch class(dist)
    case {'char', 'function_handle'}
      % A is just data
      A = a;
    case {'prmapping'}
      % A is trained mapping, dist is the application
      % of the mapping and convertin dataset to double
      A = a*dist;
      dist = @(x, y) +prmap(x, y);
    otherwise
      error('Wrong type of the DIST argument');
  end
  
  if (nargin < 2  || isempty(K))
    K = intgeos(2,1.21,m/5); % default set of nb-sizes (neighborhood-sizes)
  end
  nk = numel(K);          % number of neighborhood sizes (clusterings)
  batchsize = 100;        % batch sizes (avoid large distance matrices)
  nbatch = ceil(m/batchsize); % number of batches
  
  % first loop over the data: compute densities for every nb-size
  t = sprintf('Computing %i densities: ',m);
  prwaitbar(m,t);
  for j=1:nbatch
    prwaitbar(m,j*batchsize,[t num2str(j*batchsize)]);
    batch  = getbatch(j,batchsize,m);
    prwaitbar(m,(j-1)*batchsize,[t num2str((j-1)*batchsize)]);
    dj = feval(dist,a(batch,:),A); % distances between batch and all data
    dd = sort(dj,2);               % sort them per object
    fbatch = 1./dd(:,K);           % density estimates for all nb-sizes
    savefbatch(j,fbatch);
    %disp([j,size(fbatch)])
  end
  prwaitbar(0);
  
  % second loop: rearrange object oriented batch files 
  %              into neigborhoodsize oriented files 
  for n=1:nk                       % run over all neighborhoodsizes
    fn = zeros(m,1);               % density per object for given n
    for j=1:nbatch                 % run over all batches
      batch  = getbatch(j,batchsize,m);
      %disp([n,j])
      fbatch = readfbatch(j);      % read from file
      fn(batch) = fbatch(:,n);     % collect for single neighborhoodsize
    end
    savendens(n,fn);              % save densities for this neighborhoodsize
  end
  
  % third loop: find indices of maximum density point in every nb
  NNp = zeros(m,1);              % index of point with max density
  for n=1:nk
    savenpoint(n,NNp);
  end
  t = sprintf('Checking %i neighborhoods: ',m);
  prwaitbar(m,t);
  for j=1:nbatch
    prwaitbar(m,j*batchsize,[t num2str(j*batchsize)]);
    batch  = getbatch(j,batchsize,m);
    mj = numel(batch);             % number of data points in batch
    dj = feval(dist,a(batch,:),A); % recomputes distances
    [~,J] = sort(dj,2);            % sort them per object
    for n=1:nk                     % run over all nb's
      fn = readndens(n);          % densities for this nb-size
      NNp = readnpoint(n);
      [~,I] = max(reshape(fn(J(:,1:K(n))),mj,K(n)),[],2);
      NNp(batch)  = J([1:mj]'+(I-1)*mj); % for all points the index of
      savenpoint(n,NNp);
    end                            %  point with maximum local density 
  end
  prwaitbar(0);
  
  % fourth loop: follow gradient to density modes
  t = sprintf('Computing %i clusterings: ',nk);
  prwaitbar(nk,t);
  labels = zeros(m,nk);
  for n=1:nk
    prwaitbar(nk,n,[t num2str(n)]);
    N = readnpoint(n);                % initial nb points with maximum density
    % Re-assign samples to the sample their nearest neighbour is assigned to.
    % Iterate until assignments don't change anymore. Samples that then point 
    % to themselves are modes; all other samples point to the closest mode.
    M = N(N);
    while (any(M~=N))
      N = M; M = N(N);
    end
    labels(:,n) = M;  % labels are the indices of the modes to which data 
                      % points are assigned to. 
    % Data points assigned to the same mode constitute a cluster
  end
  prwaitbar(0);

return

function batch = getbatch(j,batchsize,m)
  % find object indices of batch j
  batch = (j-1)*batchsize+1:j*batchsize;  % take a subset of the objects
  if any(batch>m)              % arrange last batch
    batch = batch(batch <= m);
  end
return

function savefbatch(i,data)
  save(fullfile('fbatch',['file_' num2str(i,'%04i')]),'data');
return

function savendens(i,data)
  save(fullfile('ndens',['file_' num2str(i,'%04i')]),'data');
return

function savenpoint(i,data)
  save(fullfile('npoint',['file_' num2str(i,'%04i')]),'data');
return

function data = readfbatch(i)
  s = load(fullfile('fbatch',['file_' num2str(i,'%04i') '.mat']));
  data = s.data;
return

function data = readndens(i)
  s = load(fullfile('ndens',['file_' num2str(i,'%04i') '.mat']));
  data = s.data;
return

function data = readnpoint(i)
  s = load(fullfile('npoint',['file_' num2str(i,'%04i') '.mat']));
  data = s.data;
return
