%MODECLUSTF Fast KNN mode-clustering, based on overlaping cells
%
%   [LAB,NNLAB,NDIST] = MODECLUSTF(A,C,K,NEST)
% 
% INPUT
%   A       Dataset of M objects.
%   C       Integer, complexity parameter 1 <= C <= M, default C = 2.
%   K       Vector with neighborhood sizes of interest, default is a
%           geometric series between 1 and M.
%   NEST    Logical, if TRUE the output set of clusterings (columns of LAB)
%           is made nested by RECLUSTN. Default: TRUE.
%
% OUTPUT
%   LAB     Indices of mode samples, size [M,N] with N the number of
%           clusterings (NUMEL(K)).
%   NNLAB   [M,1} vector with indices of nearest neighbors.
%   NDIST   Total number of distance calculations.
%
% DESCRIPTION
% This is a fast version of MODECLUST, useful and essential for very large
% dataset (more than a million objects). It makes for every object a rough
% estimate of the potential set of nearest neighbors (which should be
% larger than max(K)). This set is larger for larger values of C, resulting 
% in a slower, but more accurate procedure. In many practical problems it
% appeared that C = 6 was sufficient.
%
% The computational complexity of this algorithm (number of distances that
% are actually computed) is M x SQRT(M). However, for small M it is not
% faster than MODECLUST. Therefor, it will jump to that routine for small
% values of M as well as for C == 0.
%
% REFERENCE
% R.P.W. Duin and S. Verzakov, Fast kNN mode seeking clustering applied to
% active learning, arXiv:1712.07454, 2017, 1-23. 
% 
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MAPPINGS, DATASETS, DISTM, PROXM, CLUSTM, DCLUSTM, MODECLUST_BATCH, 
% MODECLUST, RECLUSTN

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [lab,T,ndist] = modeclustf(varargin)

  argin = shiftargin(varargin,'scalar',1);
  argin = setdefaults(argin,[],2,[],true); 
  % defaults: c = 2, nest = false
  if mapping_task(argin,'definition')
    % definition of the mapping
    lab = define_mapping(argin,'fixed');
  else
    [a,c,K,nest] = deal(argin{:});
    % a   : original dataset
    % c   : complexity parameter, i.e. cell overlap in kNN search with k=c
    % K   : vector with k-values in kNN mode seeking
    % nest: logical, true if nested outcome is desired.
    % get rid of PRTools overhead and avoid identical objects
    a = +a.*(ones(size(a))+randn(size(a)).*1e-6);
    m = size(a,1);
    isemptyK = isempty(K);
    if c == 0 && isemptyK && prmem(m,m) == 1 % jump to dclustm if possible
      d = sqrt(distm(a));
      [lab,K] = dclustm(d,[],nest);
      [~,nnlab] = sort(d,2);
      T = nnlab(:,2);
      return
    end
    if c == 0               % jump to modeclust if desired ...
      [lab,T,K,ndist] = modeclust(a,K,[],nest); % or for small datasets
      return
    end
    n = round(sqrt(m*c));   % number of cells to be created
    R = randperm(m,n);      % by a NN proc of a random subset
    x = a(R,:);             % the initial subset
    ndist = m*n;            % keep track of number of computations
                            
    [P,Q,r] = createcells(a,x,c);
    n = numel(P);
    ndist = ndist+m*n;      % distances are recomputed for reduced cells

    if isempty(K)
      K = intgeos(3,1.21,r); % nb sizes used for computing densities
    else
      K = K(K<r);
    end
    
    nk = numel(K);           % number of nb sizes (clusterings)
    f = zeros(m,nk);         % space for storing density estimates per nb-size

    % first loop over the data: compute densities for every nb-size
    t = sprintf('Computing %i densities: ',m);
    prwaitbar(2*m+nk,t);
    nobj = 0;
    for j=1:n
      % the next loop is needed to keep array sizes within PRMEMORY
      [loops,L] = prbatch(numel(P{j}),numel(Q{j}));
      for i=1:loops
        nobj = nobj+numel(L{i});
        prwaitbar(2*m+nk,nobj,[t num2str(nobj)]);
        dji = +distm(a(P{j}(L{i}),:),a(Q{j},:));% distances in cell
        dd = sort(dji,2);                 % sort them per object
        f(P{j}(L{i}),:) = 1./dd(:,K);     % density estimates for all nb-sizes
        ndist = ndist+numel(+dji);
      end
    end  

    % second loop: find indices of maximum density point in every nb
    N = zeros(m,nk);              % space for storing these indices
    T = zeros(m,1);               % space for first neighbors
    t = sprintf('Checking %i neighborhoods: ',m);
    nobj = 0;
    for j=1:n
      % the next loop is needed to keep array sizes within PRMEMORY
      [loops,L] = prbatch(numel(P{j}),numel(Q{j}));
      for i=1:loops
        nobj = nobj+numel(L{i});
        prwaitbar(2*m+nk,m+nobj,[t num2str(nobj)]);
        dji = distm(a(P{j}(L{i}),:),a(Q{j},:)); % recompute distances in cell
        mji = numel(L{i});
        [~,J] = sort(dji,2);             % sort them per object
        for k=1:nk                       % run over all nb's
          ff = f(Q{j},k);                % densities in this cell
          [~,I] = max(reshape(ff(J(:,1:K(k))),mji,K(k)),[],2);
          N(P{j}(L{i}),k)=Q{j}(J([1:mji]'+(I-1)*mji)); % for all objects the index of
        end                              %  object with maximum local density 
        T(P{j}(L{i}),1) = Q{j}(J(:,2));
        ndist = ndist+numel(+dji);
      end
    end  

    % third loop: follow density gradient to modes
    lab = zeros(m,nk);
    t = sprintf('Computing %i clusterings: ',nk);
    for i=1:nk
      prwaitbar(2*m+nk,2*m+i,[t num2str(i)]);
      L = N(:,i);                 % initial nb objects with maximum density
      % Re-assign objects to the object their nearest neighbour is assigned to.
      % Iterate until assignments don't change anymore. Objects that point 
      % to themselves are modes; all other objects point to the cluster mode.
      if isempty(L) || any(L<1)
        break;
      end
      M = L(L);
      while (any(M~=L))
        L = M; M = L(L);
      end
      lab(:,i) = M;     % labels are the indices of the modes to which 
                        % objects are assigned to. 
      % Objects assigned to the same mode constitute a cluster
    end
    prwaitbar(0);
    if nest, lab = reclustn(lab,T); end
    if isemptyK
      lab = clustext(a,lab);
      if nest, lab = reclustn(lab,T); end
    end
    lab = labsort(lab,'up');
  end
return

%CREATECELLS

function [P,Q,r] = createcells(a,x,c,first)

% this routine is called twice, once from outside and once from inside
% after too small cells have been removed.
if nargin < 4, first = true; end
n = size(x,1);     % #objects generating the cells
m = size(a,1);     % total #objects
P = cell(1,n);     % space for primary cells
Q = cell(1,n);     % space for secondary cells (will overlap)
L = labnn(a,x,c);  % kNN procedure with k=c to create the cells
Z = true(1,n);     % track small or empty cells (caused by identical objects)
q = zeros(1,n);    % sizes of Q cells
for j=1:n          % find the n cells
  P{j} = find(L(:,1)==j); % the primary cell
  if first && isempty(P{j}), 
    Z(j) = false;  % remove empty cells
  end
  Q{j} = P{j};     % the secondary (overlap) cells
  for i=2:c        % add objects outside primary cells
    Q{j} = [Q{j}; find(L(:,i)==j)];
  end
  q(j) = numel(Q{j});     % cell size
  if first && (numel(Q{j}) < m/(3*n)), Z(j) = false; end
end
if any(~Z) % remove objects that cause too small cells and repeat once.
  [P,Q,r] = createcells(a,x(Z,:),c,false);
else
  r = min(q);
end

%CLUSTEXT Extend the set of clusterings with lower level clusterings.

function labout = clustext(a,labin)

labin = labsort(labin,'down');
% Find the largest one smaller that 10*the smallest
S = clustnum(labin);
n = find(S < 10*min(S),1);
if isempty(n) % otherwise take the smallest one
  n = numel(S);
end
% store the cluster prototypes in S
[U,~,IU] = unique(labin(:,n)); % IS will index them
labs = modeclust(a(U,:));      % modeseek cluster the selected prototypes
% apply the clusterings of the prototypes to all objects they represent
labext = zeros(size(labin,1),size(labs,2));
for i=1:size(labs,2)   
  labext(:,i) = U(labs(IU(labin(:,n)),i));
end
labout = labsort([labin labext],'downred'); % extend the multi-level clustering
labout = fliplr(labout);


