%NNPROTOC Nearest neighbor classifier based on cluster prototypes
%
%  W = NNPROTOC(LABC,A)
%  W = LABC*NNPROTOC(A)
%  LABOUT = B*W*LABELD
%
% INPUT
%   LABC    Index vector, indices of cluster prototypes for M objects.
%   A       Feature based dataset or double array with M objects (rows) 
%           used for the clustering obtaining LABC.
%   B       Test objects for which clusters should be found
%
% OUTPUT
%   W       Trained classifier based on the 1NN rule using the prototypes.
%   LABOUT  Pointers of the objects in B to the cluster prototypes of A.
%
% DESCRIPTION
% By this routine new objects B may assigned to the clusters found for
% another dataset, A. It uses the 1-nearest neighbor (NN) rule based on the
% prototypes found by some clustering applied to A. This is fine for
% spherical procedures like kmeans (CLUSTK). The prototypes generated by
% procedures like exemplar (CLUSTE), modeseeking (CLUSTM) or mean shift
% (CLUSTS) do not represent the cluster shape will. High resolution
% clusterings are advised for such procedures.
%
% There is a specific classifier for hierarchical clustering, CLUSTHC. wich
% uses the linkage type on which the clusters are based.
%
% These routines generate cluster labels. In case (some) true class labels
% are given semi-supervised classifiers can be used, see SEMISUPC and
% CLUSTC.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, KNNC, CLUSTE, CLUSTH, CLUSTK, CLUSTM, CLUSTS, CLUSTHC

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function w = nnprotoc(varargin)

  argin = shiftargin(varargin,'last',1);
  argin = setdefaults(argin,[],[]);
  if mapping_task(argin,'definition')
    w = define_mapping(argin,'fixed');
  else
    [labc,a] = deal(argin{:});
    if size(labc,1) ~= size(a,1)
      error('Clustering and data should have the same number of objects')
    end
    N = labc*clustnum;        % find number of clusters per clustering
    [~,n] = max(N);           % use the most high resolution one
    L = unique(labc(:,n));    % find it prototypes
    p = prdataset(a(L,:),L);  % dataset using prototype indices as labels
    w = fnnc(p);              % creat 1-nn classifier
  end
  
return
    



