%PRECLUST Pre-cluster to reduce dataset, cluster and merge
%
%   LAB = PRECLUST(A,CLUSTT,CLUSTP,MSIZE)
%
% INPUT
%   A     Feature based dataset or double array with M objects (rows).
% CLUSTT  PRTools mapping of target clustering routine asking for
%         clusterings of sizes K
% CLUSTP  PRTools mapping of preclustering routine
%  MSIZE  Number of objects (M) above which the dataset is preclustered by
%         CLUSTM, reducing it to MSIZE objects. Default MSIZE = 5000.
%
% OUTPUT  
%   LAB   M*NUMEL(K) array with the results of the multilevel clustering 
%         for the M objects. The columns refer to the clusterings. They 
%         yield for the objects the prototype indices of the clusters they
%         belong to.
%
% DESCRIPTION
% This routine enables various target cluster procedures like CLUSTK
% to use very large datasets (e.g. 5000 to 10^6 objects) that are
% prohibitive otherwise. First a preclustering CLUSTP is performed by a
% routine that can handle such datasets, e.g. CLUSTM. Nest, the prototpyes
% found by this dataset (at most MSIZE) are then used by the target cluster
% routine CLUSTT. Finally the clusters found by CLUSTP are assigned to the
% clusters of their prototypes determined by CLUSTT.
%
% Thsi routine is called by CLUSTK, CLUSTH, 
%
% EXAMPLE
%               % Generate 10^5 2D objects, 10 clusters
% data = gendatclust1(100000);
%               % PRTools mapping of CLUSTH asking for 5 clusterings
%               % of [2 5 10 17 30] clusters by Single Linkage 
% clustt = clusth([],[2 5 10 17 30]);
%               % PRTools mapping of fast modeseeking clustering
% clustp = clustm;
%               % Run PRECLUST, use at most 1000 prototypes for CLUSTK
% lab = preclust(data,clustt,clustp,1000);
%               % Scatterplot of data showing 10 clusters
% figure; scatn(lab(:,3),data);
%               % Learning curve for active learning
% figure; clusteval(lab,data); fontsize(12)
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, CLUSTK, CLUSTH, CLUSTS, CLUSTE, CLUSTM, CLUSTEVAL

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function lab = preclust(varargin)

  argin = shiftargin(varargin,{'prmapping'},1);
  argin = setdefaults(argin,[],[],clustm,5000);
  if mapping_task(argin,'definition')
    lab = define_mapping(argin,'fixed');
  else
    [a,clust2,clust1,msize] = deal(argin{:});
    lab1 = a*clust1;                     % run pre-clustering
    lab1 = labsort(lab1,'up');           % sort
    nlab = clustnum(lab1);               % nr of clusters / clustering
    flab = find(nlab < msize,1,'last');  % first one < msize clusters
    prot = unique(lab1(:,flab));         % corresponding prototypes    
    lab2 = a(prot,:)*clust2;             % final clustering of prototypes
    lab  = zeros(size(a,1),size(lab2,2));% space for result
    for i=1:size(lab2,2)                 % run over all levels
      lab(:,i) = lab1(:,flab);           % get high resolution lab1 results
      lab(prot,i) = prot(lab2(:,i));     % merge lab2 results of prototypes
      lab(:,i) = lab(lab(:,i),i);        % iterate
    end
    klab = clustnum(lab);
    lab = [lab lab1(:,nlab > max(klab))];
  end