%RECLUSTH Hierarchical reclustering of a multilevel clustering
%
%  [LABOUT,DEN] = RECLUSTH(LABIN,K,TYPE)
%  [LABOUT,DEN] = LABIN*RECLUSTH(K,TYPE)
%
% INPUT
%  LABIN  Multilevel labeling, size [M,L], for M objects.
%   K     Vector with desired numbers of clusters, default sampling of [2:M]
%  TYPE   Linkage rule: 's', 'a', 'r' or 'c'. Alternatively
%         'single','average','central' or 'complete'. Default 'complete'.
%
% OUTPUT
%  LABOUT Index array, size [M,length(K)], indices of cluster prototypes.
%         Columns refer to different clusterings and are ranked to
%         increasing numbers of clusters.
%  DEN    Dendrogram, see PLOTDG or DENDROGRAM
%
% DESCRIPTION
% This routine performs an agglomorative hierarchical clustering based on a
% distance matrix between all clusters on the highest level of LABIN. All 
% clusterings are used to compute the distances based on histograms of
% cluster assignments of all objects in a cluster. 
%
% The quality of this routine may be improved considerably by using a very
% diverse set of input labelings LABIN.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTH, CLUSTE, CLUSTF CLUSTK, CLUSTH, CLUSTKH, 
% CLUSTM, RECLUSTK, RECLUSTN, CLUSTEVAL, CLUSTCERR, CLUSTC, CLUSTNUM, 
% PLOTDG

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function [labout,den] = reclusth(varargin)

  argin = shiftargin(varargin,{'vector','char'},1);
  argin = shiftargin(argin,'char',2);
  [labin,K,type] = setdefaults(argin,[],[],'c');
  if mapping_task(argin,'definition')
    labout = prmapping(mfilename,'fixed',{K,type});
  else
    labin = labsort(labin,'up');
    labn  = labin(:,end);   % the basic clustering
    [x,~,L] = unique(labn); 
    m = size(x,1);          % its number of clusters
    d = zeros(m,m);         % space for the distance matrix
    K = setdefaults(K,intgeos(2,1.21,m/2)); 

    % compute distances
    for i=1:size(labin,2)-1 % add contributions by all clusterings
      [~,~,Ly] = unique(labin(:,i));
      c = numel(unique(labin(:,i)));
      z = zeros(m,c);
      for j=1:m             % use histogram bins as features
        z(j,:) = histc(Ly(labn==x(j)),[1:c]);
      end
      d = d+distm(z*normm);   % normalise per object
    end

    [labh,den] = dclusth(d,K,type);
    labout = zeros(numel(labn),numel(K));
    % find clustering per level
    for j=1:numel(K)
      labout(:,j) = x(labh(L,j));
    end
  end
