%RECLUSTK Recluster multilevel clustering to determine exactly K clusters
%
%  LABOUT = RECLUSTK(LABIN,K,LABREF)
%
% INPUT
%  LABIN  [M,L] array of indices for L clustering levels of M objects.
%         Indices should point to the prototype objects of its cluster.
%  K      Scalar or vector of length N with desired numbers of clusters.
%         Default is a set of N clusterings between 2 and M/10 clusters.
%         To reduce computing time small K's (eg K < 100) are recommended.
%  LABREF [M,1] vector of a high-resolution clustering to be used for
%         estimating cluster distances. If omitted a high-resolution
%         clustering from LABIN will be used.
%
% OUTPUT
%  LABOUT [M,N] array with the results of the multilevel clusterings of the
%         M objects. The columns refer to the N clusterings. They yield for
%         the objects the prototype indices of the clusters they belong to.
%
% DESCRIPTION
% The multilevel clustering LABIN is used to find a set / sets of exactly K
% clusters. LABREF should be a set of)high-resolution clustering(s), 
% obtaining many more clusters than demanded in K. It is used to estimate
% distances by CLUSTDIST between the clusters in a clustering in LABIN that
% has slightly more clusters than desired by K. In this clustering the 
% clusters are combined using a single linkage procedure (DCLUSTH) based on
% the computed cluster distances.
%
% This routine might be usefule for clustering procedures like CLUSTM and
% CLUSTS for which the number of desired clusters can not exactly be
% defined. 
%
% For large values of K this routine might be very time consuming.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, DCLUSTH, CLUSTE, CLUSTF CLUSTK, CLUSTH, CLUSTKH, 
% CLUSTM, RECLUSTH, RECLUSTN, TESTCLUST, CLUSTCERR, CLUSTC, CLUSTNUM, 
% PLOTDG

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function labout = reclustk(varargin)
 
  argin = shiftargin(varargin,'vector',1);
  argin = setdefaults(argin,[],[],[]);
  if mapping_task(argin,'definition')
    labout = define_mapping(argin,'fixed');
  else
    [labin,K,labref] = deal(argin{:});
    labin  = labsort(labin,'downred');
    labref = setdefaults(labref,labin(:,1));
    s = clustnum(labin); % obtained clustersizes
    if isempty(K)
      K = intgeos(2,1.21,size(labin,1)/10);
      K = K(K<100);
      K = [K s(s>=100)];
      K = K(K<min(s));
    else
      K = K(K<max(s));
    end
    if isempty(K)
      error('Demanded number of clusters not possible')
    end
    m = size(labin,1);
    K = sort(K,'descend');
    n = numel(K);
    s = clustnum(labin); % obtained clustersizes
    labout = zeros(m,n);
    t = sprintf('Finding %i clusterings: ',n);
    prwaitbar(n,t);
    for i=1:n
      prwaitbar(n,i,[t num2str(i)]);
      k = K(i);
      if any(s==k)
        labki = labin(:,s==k);
      else
        g = find(s>k,1,'last');
        [S,~,IS] = unique(labin(:,g));
        prwaitbar(n,i+0.3,[t num2str(i)]);
        dd = clustdist(labin(:,g),labref);
        prwaitbar(n,i+0.6,[t num2str(i)]);
        labd  = dclusth(dd,k,'s');
        labki = S(labd(IS(labin(:,g)),1));
      end
      labout(:,i) = labki;
    end
    prwaitbar(0);
    labout = fliplr(labout);
  end