%RECLUSTN Make multilevel clustering nested
%
%   LABOUT = RECLUSTN(LABIN)
%   LABOUT = LABIN*RECLUSTN
%
% INPUT
%   LABIN   M*N array with the results of the multi-level clusterings for M
%           objects. The columns refer to N clusterings. They yield for the
%           objects the prototype indices of the clusters they belong to.
%
% OUTPUT
%  LABOUT    Index array, size [M,length(K)], indices of cluster prototypes.
%            Columns refer to different clusterings and are ranked to
%            increasing numbers of clusters.
%
% DESCRIPTION
% Levels in a multi-level clustering are made nested: objects in the same 
% cluster of a higher resolution clustering belong to the same cluster in a
% lower-resolution clustering. This might yield an improved result as it
% combines the clustering levels.
%
% This routine is particulary useful for multilevel modeseeking clusterings
% like MEANSHIFT and MODECLUST, MODECLUSTF, CLUSTM and DCLUSTM.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% MEANSHIFT, MODECLUST, MODECLUSTF, CLUSTM, DCLUSTM, LABSORT, RECLUSTK,
% RECLUSTH

% Copyright: R.P.W. Duin, r.p.w.duin@37steps.com

function lab = reclustn(varargin)

  if numel(varargin) == 1 && min(size(varargin{1})) == 1
    argin = {[],varargin{1}};
  else
    argin = varargin;
  end
  if mapping_task(argin,'definition')
    lab = define_mapping(argin,'fixed');
  else
    [lab,nnlab] = deal(argin{:});
    if size(lab,2) < 2 % nothing to do
      return;
    end

    lab = labsort(lab,'downred');
    for n=2:size(lab,2)
      lab2  = lab(:,n);           % level_2 is this level
      lab1  = lab(:,n-1);         % level_1 is previous level
      prot1 = unique(lab1);       % level_1 prototypes
      prot2 = unique(lab2);       % level_2 prototypes
      labc  = lab1;
      labc(prot1) = lab2(prot1);  
      labc(prot2) = prot2;        % create a prototype
      labc = labc(labc);          % relabel lab1 with the level_2 prototype
      % some modes are outsize their new cluster
      J = find(lab2(lab1(prot2)) ~= prot2);  % find them
      labc(prot2(J)) = labc(lab1(prot2(J))); % correct them
      % clusters labeled with mode2(J) do not have a proper prototype
      % run over all of them to create one
      for j=1:numel(J)
        R = find(labc(prot1) == prot2(J(j)));
        L = labc == prot2(J(j));  % a level_2 cluster without a prototype
        C = cell(1,numel(R));
        S = zeros(1,numel(R));
        for i =1:numel(R)         % run over constituting level_1 clusters
          C{i} = find(lab1 == prot1(R(i)));
          S(i) = numel(C{i});
        end
        [~,imax] = max(S);        % take the largest
        labc(L) = prot1(R(imax)); % and use its level_1 prototype
      end
      lab2 = labc(labc);          % update
      lab(:,n) = lab2;             % done
    end
    lab = fliplr(lab);
  end