%SEMISUPC Semi-Supervised Classifier
%
%   [W,LABE] = SEMISUPC(T,V,U)
%   [W,LABE] = T*SEMISUPC(V,U)
%
% INPUT
%   T     Partially labeled dataset to be used for training.
%   V     Untrained classifier, default FNNC (Fast NN classifier).
%   U     Untrained clustering procedure, default MODECLUSTF([],6).
%
% OUTPUT
%   W     Trained classifier.
%   LABE  Estimated class labels of the entire dataset T.
%         LABE = T*W*LABELD
%
% DESCRIPTION
% The partially labeld dataset T should be a PRTools dataset with missing
% labels. Missing string labels have to be set to the empty string. Missing
% numeric labels have to be indicated by a NaN. Such a dataset can be
% constructed by T = [A;X]; inwhich A is a fully labeled PRTools dataset
% and X is a  matrix of doubles consisting of feature vectors in the same
% space as T
%.
% The dataset T, neglecting all labels, is first applied to the cluster 
% procedure U, which should be a PRTools fixed mapping. The resulting
% cluster indices and the known labels of T are used to estimate all object
% labels by CLUSTC. The resulting datasets is used for trainig the
% untrained classifier V.
%
% SEE ALSO (<a href="http://37steps.com/clustertools">ClusterTools Guide</a>)
% DATASETS, MAPPINGS, CLUSTE, CLUSTF, CLUSTH, CLUSTK, CLUSTKH, CLUSTM,
% MODECLUST. MODECLUSTF, MEANSHIFT, CLUSTC, CLUSTK, FNNC, LABELD

function [w,elab] = semisupc(varargin)

argin = shiftargin(varargin,'prmapping');
argin = setdefaults(argin,[],fnnc,modeclustf([],6));
[a,v,u] = deal(argin{:});
classname = [getname(v) ' - ' getname(u) ' combiner'];
if mapping_task(argin,'definition')       % define mapping
	w = define_mapping(argin,'untrained',classname);
elseif mapping_task(argin,'training')			% train a mapping
  isdataset(a);
  labt = getnlab(a);         % given, true labels
  L = labt == 0;             % for unlabeled objects labt=0
  if sum(L) == 0             % no unlabeld objects
    w = a*v;                 % all objects labeld: no clustering needed
    elab = NaN;
  else
    labk = +a*u;             % clustering
    if size(labk,2) > 1
      labk = labsort(labk,'downred');         % order cluster levels
      k = find(clustnum(labk) < sum(~L),1);   % find good cluster level
      k = setdefaults(k,0);                   % for the supervised size
      k = max(k,5);            % we need at least some levels for combining
      k = min(k,size(labk,2)); % but not more than we have
      labe = clustc(labk(:,1:k),labt);   % classify all objects
    else
      labe = clustc(labk,labt);   % classify all objects
    end
    labe(~L) = labt(~L);     % copy given labels
    w = setnlab(a,labe)*v;   % use results to build a training set and train
    if nargout > 1
      lablist = getlablist(a);
      elab = lablist(labe,:);
    end
  end
end